#! /usr/bin/env python

import logging
import os
import getpass
from peerDescriptor import PeerDescriptor
from filesMetadata import FileInfo

class AsynchCommand:
    def __init__(self, idArg):
        self.commandId = str(idArg)
        self.executed = False
        self.synchronizePeers = []
        self.allSynchronizePeers = []
        self.allReceiveSynchroPeers = []

    def initFromDict(self, dictionary):
        for key in dictionary:
            self.__dict__[key] = dictionary[key]

    def setSynchronizePeers(self, synchronizePeersArg):
        self.synchronizePeers = synchronizePeersArg
        self.allSynchronizePeers = synchronizePeersArg

    def resetSynchro(self):
        self.synchronizePeers = self.allSynchronizePeers[:]

    def getSynchronizePeers(self):
        return self.synchronizePeers

    def getId(self):
        return str(self.commandId)

    def markSynchronizePeers(self, peers):
        for item in peers:
            if item in self.synchronizePeers:
                self.synchronizePeers.remove(item)

    def synchronizeNotRequired(self):
        self.synchronizePeers = []

    def wasExecuted(self):
        return self.executed

    def markExecuted(self):
        self.executed = True

    def __str__(self):
        res = str(self.__dict__)
        return res

class FileVersionsCommand(AsynchCommand):
    def __init__(self, idArg):
        self.commandId = str(idArg)
        self.executed = False
        self.fileInfoList = None
        self.ownerId = None
        self.synchronizePeers = []
        self.allSynchronizePeers = []

    def setOwner(self, ownerId):
        self.ownerId = ownerId

    def setFileVersions(self, fileInfoList):
        self.fileInfoList = fileInfoList + []

    def initFromDict(self, dictionary):
        for key in dictionary:
            self.__dict__[key] = dictionary[key]
        infos = []
        for infoDict in self.fileInfoList:    
            info = FileInfo(infoDict['lastLocation'], infoDict['ownerId'])
            info.initFromDict(infoDict)
            infos.append(info)
        self.fileInfoList = infos

