#!/usr/bin/python

import logs
import xmlrpclib
import time
import socket
import threading
import fnmatch
import getpass
import os
from peerDescriptor import PeerDescriptor
from asynchCommand import FileVersionsCommand

class AsynchCommandClient(object):

    def __init__(self, dhtAddress, myPeerId, myPeerAddr, myPort, statsPathArg, config):
        self.myPeerId = myPeerId
        self.myPeerAddr = myPeerAddr
        self.myPort = myPort
        self.dhtAddress = dhtAddress
        self.statsPath = statsPathArg
        self.config = config

    def sendCommandForPeer(self, toPeerId, commandType, command):
        logs.debug("ASYNCH_COMMAND: sending command type: " + str(commandType) + " to peer: " + str(toPeerId) + ", with version " + command.commandId)
        try:
            toPeer = PeerDescriptor(self.dhtAddress, toPeerId)
            if self.myPeerId in toPeer.metadataLocations:
                asynchCommandKeepers = toPeer.metadataLocations + []
            else:
                asynchCommandKeepers = toPeer.metadataLocations + [self.myPeerId]
            synchroPeers = asynchCommandKeepers + [toPeerId]
            logs.debug("ASYNCH_COMMAND: synchroPeers for command: " + str(synchroPeers))
            command.setSynchronizePeers(synchroPeers)
            if len(asynchCommandKeepers) == 0:
                return False
            myProxy = xmlrpclib.ServerProxy("http://"+str(self.myPeerAddr) + ":" + str(int(self.myPort) + 1), allow_none = True)
            myProxy.persistCommandForUser(self.myPeerId, toPeerId, commandType, command)
            return True
        except socket.error as e:
            logs.commonErr(e)
            return False
        except Exception as e:
            logs.trace(e)
            return False

    def startDaemon(self):
        while True:
            time.sleep(self.config.asynchComandSynchronizationIntervalSec)
            self.periodicWakeup()

    def periodicWakeup(self):
        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myPeerAddr) + ":" + str(int(self.myPort) + 1), allow_none = True)
        myProxy.periodic_sync()


def startAsynchCommandDaemon(dhtAddress, myPeerId, myAddr, myPort, statsPathArg, config):
    asynchCommandClient = AsynchCommandClient(dhtAddress, myPeerId, myAddr, myPort, statsPathArg, config)
    asynchCommandClient.startDaemon()
