#!/usr/bin/python
import logs
import xmlrpclib
import os
import sys
import pickle
import socket
import getpass
import thread
from SimpleXMLRPCServer import SimpleXMLRPCServer
from xmlrpclib import Fault
from common import KeysManager
from filesOperations import FilesOperations
from asynchCommand import FileVersionsCommand
from peerDescriptor import PeerDescriptor
from persistentDict import PersistentDict

class AsynchCommandServer(object):
    def __init__(self, metadataPath, myPeerIdArg, myAddr, myPort, homeDirArg, dataDirArg, statsPathArg):
        self.dataDir = dataDirArg
        self.myAddr = myAddr
        self.myPort = myPort
        self.homeDir = homeDirArg
        self.myPeerId = myPeerIdArg
        self.statsPath = statsPathArg
        # this is map from commandType -> fromPeerId -> toPeerId -> (command, lastExecuted)
        self.asynchCommands = PersistentDict(metadataPath + "/asyncCommands-" + str(self.myPort)) 

    def isWorking(self):
        return True

    def persistCommandForUser(self, fromId, toId, commandType, commandDict):
        try:
            logs.info("ASYNCH_COMMAND: Received command from " + str(fromId) + " to " + str(toId) + ", with synchronization peers: " +\
                      str(commandDict['allSynchronizePeers']))
            command = None
            if commandType == 'FileVersionsCommand':
                command = FileVersionsCommand(0)
            command.initFromDict(commandDict)

            if self.asynchCommands.has_key([commandType, fromId, toId]):
                oldCommand, lastReceived = self.asynchCommands.get([commandType, fromId, toId])

                if lastReceived < command.commandId:
                    command.resetSynchro()
                    command.markSynchronizePeers([str(self.myPeerId)])
                    self.asynchCommands.set([commandType, fromId, toId], (command, command.commandId))
                else:
                    #just ignore msg that was rceived earlier
                    return 
            else:
                command.resetSynchro()
                command.markSynchronizePeers([str(self.myPeerId)])
                self.asynchCommands.set([commandType, fromId, toId], (command, command.commandId))

            self.asynchCommands.sync()

        except Exception as e:
            logs.trace(e)

    def periodic_sync(self):
        thread.start_new_thread(self.periodic_sync_thread, ())

    def periodic_sync_thread(self):
        try:
            logs.info("ASYNCH_COMMAND: Periodically woken up")
            commands = self.asynchCommands.keys([])
            for cmd in commands:
                fromPeers = self.asynchCommands.keys([cmd])
                for fromPeerId in fromPeers:
                    toPeers = self.asynchCommands.keys([cmd, fromPeerId])
                    for toPeerId in toPeers:
                        command, lastReceived = self.asynchCommands.get([cmd, fromPeerId, toPeerId])
                        if (self.myPeerId == toPeerId) and (not command.wasExecuted()):
                            succ = self.execute(cmd, command)
                            logs.info("ASYNCH_COMMAND: Executing command from peer: " + str(fromPeerId))
                            if succ:
                                command.markExecuted()
                        else:
                            syncPeers = command.synchronizePeers
                            succPeers = []
                            for s in syncPeers:
                                try:
                                    remotePeer = PeerDescriptor(self.dhtAddress, str(s))
                                    remoteProxy = xmlrpclib.ServerProxy(
                                                  "http://"+str(remotePeer.ipAddress) + ":" + str(int(remotePeer.portNumber) + 1),
                                                  allow_none = True)
                                    remoteProxy.persistCommandForUser(fromPeerId, toPeerId, cmd, command)
                                    succ = True
                                except socket.error as e:
                                    logs.commonErr(e)
                                    succ = False
                                except Exception as e:
                                    logs.trace(e)
                                    succ = False
                                if succ:
                                    succPeers.append(s)
                            command.markSynchronizePeers(succPeers)
                        self.asynchCommands.set([cmd, fromPeerId, toPeerId], (command, lastReceived))

            self.asynchCommands.sync()
        except Exception as e:
            logs.trace(e)

    def execute(self, commandType, command):
        if commandType == "FileVersionsCommand":
            remoteProxy = xmlrpclib.ServerProxy(
                              "http://"+str(self.myAddr) + ":" + str(self.myPort),
                              allow_none = True)
            remoteProxy.verifyFileVersions(command.ownerId, command.fileInfoList)
        return True

def startAsynchCommandServer(metadataPath, myId, ipAddr, port, homeDir, dataDir, statsPath, dhtAddress = None):
    asynchCommandServer = AsynchCommandServer(metadataPath, myId, ipAddr, port, homeDir, dataDir, statsPath)

    asynchCommandServer.dhtAddress = dhtAddress
    asynchCommandServer.port = port
    asynchCommandServer.ipAddress = ipAddr
    asynchCommandServer.username = getpass.getuser()
    
    rpcServer = SimpleXMLRPCServer((asynchCommandServer.ipAddress, int(asynchCommandServer.port) + 1), allow_none=True, logRequests = False)
    logs.info("ASYNCH_COMMAND: Listening on ipAddress: " + str(asynchCommandServer.ipAddress) + ", and port: " +str(int(port) + 1)+" ...")
    rpcServer.register_instance(asynchCommandServer)
    try:
        rpcServer.serve_forever()
    except KeyboardInterrupt:
        pass


