#! /usr/bin/env python

import time
import xmlrpclib
import socket
from peerDescriptor import PeerDescriptor

class Timer:
    def getTime(self):
        return time.time()

    def sleep(self, sec):
        time.sleep(sec)

class AvailabilityManager:
    def __init__(self, config, serverHttpAddr, myId, filesCatalog, timer = Timer()):
        self.config = config
        self.filesCatalog = filesCatalog
        self.timer = timer
        self.durationIgnored = config.durationIgnoredSec
        self.serverHttpAddr = serverHttpAddr
        self.myId = myId

    def periodicMonitor(self):
        while True:
            self.makeMeasurement()
            self.exposeAvailability()
            self.timer.sleep(self.config.availabilityMeasurementIntervalSec)

    def makeMeasurement(self):
        current = self.timer.getTime()
        timestamps = self.filesCatalog.getAvailabilityTimeStamps()
        self.removeOld(timestamps, current)
        timestamps.append(current)
        self.filesCatalog.setAvailabilityTimeStamps(timestamps)

    def removeOld(self, timestamps, current):
        rememberHorizont = current - self.config.availabilityMeasurementPeriodSec
        while (len(timestamps) > 0) and (timestamps[0] < rememberHorizont):
            del timestamps[0]

    def getAvailability(self):
        current = self.timer.getTime()
        measurementDur = min(self.config.availabilityMeasurementPeriodSec, current - self.filesCatalog.getAvailabilityStartTime())
        if measurementDur < self.durationIgnored:
            # to little samples
            return 1.0
        timestamps = self.filesCatalog.getAvailabilityTimeStamps()
        self.removeOld(timestamps, current)
        if len(timestamps) < 1:
            return 0.0
        previous = current - measurementDur
        totalGap = 0
        for t in timestamps:
            gap = t - previous - self.config.availabilityMeasurementIntervalSec - 60 * 5 # these 5 minues are to capture errors in synchronization
            previous = t
            if gap > 0:
                totalGap += gap

        av = (measurementDur - totalGap) / measurementDur
        
        assert(av >= 0.0)
        assert(av <= 1.0)
        return av

    def exposeAvailability(self):
        av = self.getAvailability()

        proxy = xmlrpclib.ServerProxy(self.serverHttpAddr, allow_none=True)
        stop = False
        while not stop:
            stop = True
            try:
                try:
                    myPeerDescriptor = PeerDescriptor(self.serverHttpAddr, str(self.myId))
                except xmlrpclib.Fault:
                    myPeerDescriptor = PeerDescriptor()

                proxy.putField(str(self.myId), "availability", av)
                proxy.putField(str(self.myId), "availabilityTimestamp", time.time())
                proxy.putField(str(self.myId), "availabilityMeasurementDuration", self.config.availabilityMeasurementPeriodSec)
            except socket.error:
                stop = False
                print "Trying to connect once again"
                time.sleep(2)


