#! /usr/bin/env python

import os
import getpass
import hashlib
import time
import pickle
import marshal
import types
import heapq
import config
from datetime import date

sizeScalingFactor = 1 #1024 * 1024

if config.enviroment == "home":

    def remoteCommand(user, host, direc, command, logfile = None):
        if logfile == None:
            remoteCmd = "ssh -f -o ConnectTimeout=3 " + user + "@" + host + " \" cd " + direc + "; " + command + " \""
        else:
            remoteCmd = "ssh -f -o ConnectTimeout=3 " + user + "@" + host + " \" cd " + direc + "; " + command + " > " + logfile + " 2>&1 \""
        os.system(remoteCmd)
else:

    def remoteCommand(user, host, direc, command, logfile = None):
        if logfile == None:
            remoteCmd = "ssh -n -f -o ConnectTimeout=3 " + user + "@" + host + " \" cd " + direc + "; sh -c \\\" " + command + " > /dev/null 2>&1 &\\\"\""
        else:
            remoteCmd = "ssh -n -f -o ConnectTimeout=3 " + user + "@" + host + " \" cd " + direc + "; sh -c \\\" " + command + " > " + logfile + " 2>&1 &\\\"\""
        os.system(remoteCmd)

def calcResiliencyLevel(filename):
    return 3

class KeysManager:
    def __init__(self, hostname, homeDirArg, userPathArg):
        self.hostname = hostname
        self.homeDir = homeDirArg
        self.userPath = userPathArg
        self.username = getpass.getuser()
        self.privateKeyFile = os.path.join(self.homeDir, ".ssh", "id_rsa")
        self.publicKeyFile = os.path.join(self.homeDir, ".ssh", "id_rsa.pub")
        self.authorizedKeysFile = os.path.join(self.homeDir, ".ssh", "authorized_keys")

    def generateKeys(self):
        if not (os.path.isfile(self.privateKeyFile) and os.path.isfile(self.publicKeyFile)):
            keyGenCommand = "ssh-keygen -t rsa -P \"\" -f %s > /dev/null" % self.privateKeyFile
            os.system(keyGenCommand)

    def getPublicKey(self):
        self.generateKeys()
        f = open(self.publicKeyFile, 'r')
        publicKey = f.readline()
        f.close()
        return publicKey

    def authorizeKey(self, key):
        if (config.enviroment == "lab") or (config.enviroment == "planetLab"):
            pass
        else:
            alreadyAuthorized = False
            if os.path.isfile(self.authorizedKeysFile):
                fRead = open(self.authorizedKeysFile, 'r')
                for line in fRead:
                    if line == key:
                        alreadyAuthorized = True
                fRead.close()
            if not alreadyAuthorized:
                fWrite = open(self.authorizedKeysFile, 'a')
                fWrite.write(key)
                fWrite.close()

    def getPeerId(self):
        if config.enviroment == "home":
            return self.username
        elif (config.enviroment == "lab") or (config.enviroment == "planetLab"):
            return self.hostname
        else:
            md5 = hashlib.md5()
            with open(self.privateKeyFile, 'rb') as f: 
                for chunk in iter(lambda: f.read(8192), ''): 
                    md5.update(chunk)
            md5.update(self.userPath)
            return str(int(md5.hexdigest(), 16) % 1000000000000000)

class QueueWithTimeouts:
    def __init__(self):
        self.queue = {}

    def add(self, obj, timeout):
        if not self.queue.has_key(obj):
            self.queue[obj] = time.time() + timeout

    def has(self, obj):
        if not self.queue.has_key(obj):
            return False
        current = time.time()
        if (current > self.queue[obj]):
            self.queue.pop(obj)
            return False
        return True

class FunctionArgsWithDelays:
    def __init__(self, filename, callback):
        self.filename = filename
        self.args = []
        self.callback = callback
        if os.path.exists(self.filename):
            pickleFile = open(self.filename, 'rb')
            self.args = pickle.load(pickleFile)
            pickleFile.close()

    def sync(self):
        pickleFile = open(self.filename, 'wb')
        pickle.dump(self.args, pickleFile)
        pickleFile.close()

    def put(self, args, timeout):
        deadline = time.time() + timeout
        heapq.heappush(self.args, (deadline, args))

    def run(self):
        current = time.time()
        if len(self.args) < 1:
            return
        (deadline, args) = self.args[0]
        while deadline < current:
            heapq.heappop(self.args)
            self.callback(*args)
            if len(self.args) < 1:
                self.sync()
                return
            (deadline, args) = self.args[0]
        self.sync()

def funcWithDelaysTest():
    def printer(*args):
        for a in args:
            print a

    waiting = FunctionArgsWithDelays("testFile", printer)
    waiting.sync()
    waiting.put(["a", "bb"], 1)
    waiting.put(["c", "d"], 1)
    waiting.sync()
    waiting.run()

if config.enviroment == "home":
    def day_difference(t1, t2):
        if t2 - t1 > 60:
            return 1
        else:
            return 0
else:
    def day_difference(t1, t2):
        d1 = date.fromtimestamp(t1)
        d2 = date.fromtimestamp(t2)
        dif = d2 - d1
        res = dif.days
        return res


def get_size(start_path):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            total_size += os.path.getsize(fp)
    return total_size

def acquire(l):
    l.acquire()

def release(l):
    try:
        l.release()
    except:
        pass

    #os.system("rm testFile")


#if __name__ == '__main__':
#    funcWithDelaysTest()

