#! /usr/bin/env python

import hashlib
import xmlrpclib
import shelve
import random
import os
import logs
import time
import socket
from peerDescriptor import PeerDescriptor
from persistentDict import PersistentDict

#def computeFileId(filename):
    #"""
    #given a filename, computes its ID by hashing file content
    #"""
    #chunksize = 1 * 2**20

    #try:
    #    binFile = open(filename, 'rb')
    #except IOError:
    #    return -1
    #hashAlg = hashlib.sha1()
    #while (True):
    #    chunk = binFile.read(chunksize)
    #    if chunk == "":
    #        break
    #    hashAlg.update(chunk)
        
    #binFile.close()
    #hashValue = hashAlg.hexdigest()
    #return hashValue

def computeFileId(filename):
    hashAlg = hashlib.sha1()
    hashAlg.update(filename)
    hashValue = hashAlg.hexdigest()
    return hashValue

class FileInfo:
    """
    Information about a single file; 
    """
    def __init__(self, filename, ownerIdArg):
        self.replicatorsIds = []
        self.committedLocation = [] #this is a list (commitedLocation, time)
        self.oldLocations = []
        self.size = 0
        self.lastLocation = filename
        self.fileId = computeFileId(filename)
        self.version = 1
        self.ownerId = str(ownerIdArg)
        # TODO: what if there are two files with same content? id is not unique

    def initFromDict(self, dictionary):
        for key in dictionary:
            self.__dict__[key] = dictionary[key]
        self.size = int(self.size)
        self.version = int(self.version)
    def __repr__(self):
        return "loc: " + str(self.lastLocation) + ", replicators: " + str(self.replicatorsIds) + " ,size: " + str(self.size)


def serializeFileInfo(fileInfo):
    newFileInfo = FileInfo(fileInfo.lastLocation, fileInfo.ownerId)
    newFileInfo.replicatorsIds = fileInfo.replicatorsIds
    newFileInfo.oldLocations = fileInfo.oldLocations
    newFileInfo.size = str(fileInfo.size)
    newFileInfo.version = str(fileInfo.version)
    return newFileInfo

def initFromDictIfRequired(fileInfo):
    try:
        fileInfo.items()
        res = FileInfo(fileInfo['lastLocation'], fileInfo['ownerId'])
        res.initFromDict(fileInfo)
        return res
    except AttributeError:
        return fileInfo

class FilesCatalog:

    def __init__(self, shelfName, myAddr, myPortNum, filesOperations = None):
        """
        Initializes new object FilesCatalog based on persistent info from shelve shelfName; shelfName must be absolute path
        """
        self.filesOperations = filesOperations
        self.shelfName = shelfName
        self.myId =  self.filesOperations.myPeerId
        self.myAddr = myAddr
        self.myPortNum = myPortNum
        self.myNegoProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(self.myPortNum), allow_none = True)

        self.dirty = False

        self.storage = PersistentDict(self.shelfName)
        self.filesOperations.filesCatalog = self

        if (not self.storage.has_key(['availabilityTimeStamps'])):
            self.storage.set(['availabilityTimeStamps'], [])
        if (not self.storage.has_key(['availabilityStartTime'])):
            self.storage.set(['availabilityStartTime'], time.time())
        if self.storage.has_key(['myFirstFile']):
            self.myFirstFile = self.storage.get(['myFirstFile'])
        else:
            self.myFirstFile = None

        self.storage.sync()

    def getAvailabilityTimeStamps(self):
        return self.storage.get(['availabilityTimeStamps'])

    def setAvailabilityTimeStamps(self, availabilityTimeStamps):
        self.storage.set(['availabilityTimeStamps'], availabilityTimeStamps)
        self.dirty = True
        self.sync()

    def getAvailabilityStartTime(self):
        return self.storage.get(['availabilityStartTime'])

    def put(self, fileInfo):
        self.storage.set([fileInfo.fileId], fileInfo)
        if self.myFirstFile == None:
            self.myFirstFile = fileInfo.lastLocation
            self.storage.set(['myFirstFile'], self.myFirstFile)

        if fileInfo.lastLocation == self.myFirstFile:
            proxy = xmlrpclib.ServerProxy(self.filesOperations.dhtAddress, allow_none=True)
            try:
                myPeerDescriptor = PeerDescriptor(self.filesOperations.dhtAddress, str(self.myId))
            except xmlrpclib.Fault:
                myPeerDescriptor = PeerDescriptor()
                proxy.put(str(self.myId), myPeerDescriptor)
            proxy.putField(str(self.myId), "metadataLocations", fileInfo.replicatorsIds)
        self.dirty = True

    def get(self, fileId):
        return initFromDictIfRequired(self.storage.get([fileId]))

    def find(self, lastLocation):
        #TODO: probably inefficient
        """returns FileInfo based on lastLocation"""
        for key in self.storage.keys([]):
            if key not in ['myFirstFile', 'availabilityStartTime', 'availabilityTimeStamps']:
                val = initFromDictIfRequired(self.storage.get([key]))
                if val.lastLocation == lastLocation:
                    return val
        raise IndexError

    def getStorageMap(self):
        storageMap = {}
        for key in self.storage.keys([]):
            if key not in ['myFirstFile', 'availabilityStartTime', 'availabilityTimeStamps']:
                val = initFromDictIfRequired(self.storage.get([key]))
                storageMap[key] = val
        return storageMap

    def getAllFileInfo(self):
        result = []
        for key in self.storage.keys([]):
            if key not in ['myFirstFile', 'availabilityStartTime', 'availabilityTimeStamps']:
                val = initFromDictIfRequired(self.storage.get([key]))
                result.append(val)
        return result

    def sync(self):
        self.storage.sync()
        self.dirty = False


    def close(self):
        self.storage.sync()
        self.storage.close()
                  
