#! /usr/bin/env python

import pygtk
import shelve
import gtk, gobject
import time
import thread
import traceback
import xmlrpclib
import os
import re
import sys
from threading import Thread

configurationFileName = "configuration-persistent-data"
machinesMonitorSleepSec = 20

class FileSelector:
    def __init__(self):
        self.dialog = gtk.FileChooserDialog("Open..",
                               None,
                               gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                               gtk.STOCK_OK, gtk.RESPONSE_OK))
        self.dialog.set_default_response(gtk.RESPONSE_OK)
        self.dialog.unselect_all()

    def show(self):
        response = self.dialog.run()
        if response == gtk.RESPONSE_OK:
            self.save_selection()
            self.dialog.destroy()
        elif response == gtk.RESPONSE_CANCEL:
            self.dialog.destroy()

    def save_selection(self):
        print self.dialog.get_filenames()

class Machine:
    def __init__(self):
        self.machineName = ""
        self.hostname = ""
        self.port = ""
        self.username = ""
        self.homeDir = ""
        self.storageDir = ""
        self.daemonized = False
        self.directoriesForBackup = []
        self.mainPath = ""
        self.totalIncommingBwBound = 1
        self.desiredBackWindow = 0
        self.totalSpace = 1000

    def copy(self, newMachine):
        self.machineName = newMachine.machineName
        self.hostname = newMachine.hostname
        self.port = newMachine.port
        self.username = newMachine.username
        self.homeDir = newMachine.homeDir
        self.storageDir = newMachine.storageDir
        self.daemonized = newMachine.daemonized
        self.directoriesForBackup = newMachine.directoriesForBackup
        self.mainPath = newMachine.mainPath
        self.totalIncommingBwBound = newMachine.totalIncommingBwBound
        self.desiredBackWindow = newMachine.desiredBackWindow
        self.totalSpace = newMachine.totalSpace

    def __str__(self):
        return "Machine name = " + str(self.machineName) + ", hostname = " + str(self.hostname) + ", port = " + str(self.port) + \
            ", username = " + str(self.username) + ", homeDir = " + str(self.homeDir) + ", storageDir = " + str(self.storageDir) + \
            ", directoriesForBackup = " + str(self.directoriesForBackup) + ", mainPath = " + str(self.mainPath) + \
            ", totalIncommingBwBound = " + str(self.totalIncommingBwBound) + ", desiredBackWindow = " + str(self.desiredBackWindow) + \
            ", totalSpace = " + str(self.totalSpace) + ", daemonized = " + str(self.daemonized)

class MachineObjects:
    def __init__(self):
        self.checkbox = None
        self.statusImage = None
        self.daemonImage = None
        self.proxy = None

class PingAgent(Thread):
    def __init__(self, host, dic, index):
        Thread.__init__(self)   
        self.host = host
        self.dic = dic
        self.index = index

    def markAvailability(self, val):
        proxy = xmlrpclib.ServerProxy(self.dhtAddr, allow_none=True)
        proxy.markAvailability(str(self.peerId), val)

    def run(self):
        out = os.popen('ping -c 3 ' + str(self.host))
        output = out.read()
        rttS = re.search('time=([0-9]*)', output)
        ttlS = re.search('ttl=([0-9]*)', output)
        if rttS != None and ttlS != None:
            self.dic[self.index] = "OK"
        else:
            self.dic[self.index] = "UNREACHABLE"

class MachinesMonitor:
    def __init__(self, machines, machinesObjects, window):
        self.machines = machines
        self.machinesObjects = machinesObjects
        self.window = window
        self.status = {}

    def run(self):
        try:
            while True:
                self.periodicWakeup()
                time.sleep(machinesMonitorSleepSec)
        except Exception as e:
            traceback.print_exc()
            print e
            raise e

    def start_machine(self, machineName, knownPeers):
        try:
            m = self.machines[machineName]
            command = "ssh -f " + m.username + "@" + m.hostname + " mkdir -p " + m.mainPath
            os.system(command)
            command = "scp *py " + m.username + "@" + m.hostname + ":" + m.mainPath
            os.system(command)
            command = "ssh -f " + m.username + "@" + m.hostname + " python -u '" + m.mainPath + "/mainFacade.py " + m.hostname + " " + \
                str(m.port) + " " + m.homeDir + " " + m.storageDir + " " + m.storageDir + "/statistics.txt " + str(m.totalIncommingBwBound) + " " +\
                str(m.desiredBackWindow) + " " + str(m.totalSpace) + " " + m.storageDir + "/metadata " + str(knownPeers) + "'"
            os.system(command)
        except Exception as e:
            traceback.print_exc()
            print e

    def stop_machine(self, machineName):
        m = self.machines[machineName]
        os.system ("ssh " + m.username + "@" + m.hostname + " pkill -U " + m.username + " -f .*python.*")

    def mark_for_backup(self, machineName):
        m = self.machines[machineName]
        proxy = self.machinesObjects[machineName].proxy
        if proxy is None:
            print "Proxy for machine " + str(machineName) + " is None"
        else:
            for d in m.directoriesForBackup:
                proxy.backupDirectory(d)

    def delete_all_data(self, machineName):
        m = self.machines[machineName]
        command = "ssh -o ConnectTimeout=3 " + m.username + "@" + m.hostname + " rm -rf " + m.storageDir + " " + m.mainPath
        for d in m.directoriesForBackup:
            command += (" " + d)
        os.system(command)

    def print_placement(self, machineName):
        proxy = self.machinesObjects[machineName].proxy
        if proxy is None:
            print "Proxy for machine " + str(machineName) + " is None"
        else:
            proxy.printReport()

    def create_initial_data(self, machineName, size):
        dataChunkSize = 1024
        m = self.machines[machineName]
        for d in m.directoriesForBackup:
            remainSizeInDir = (size * 1024) / len(m.directoriesForBackup)
            createDirectoryCommand = "ssh -f " + m.username + "@" + m.hostname + " mkdir -p %s" % d
            os.system(createDirectoryCommand)
            fileNum = 0
            while remainSizeInDir > 0:
                fileSize = min(dataChunkSize, remainSizeInDir)
                remainSizeInDir -= fileSize
                createFileCommand = "ssh -f " + m.username + "@" + m.hostname + " dd if=/dev/zero of=%s/file%s bs=%s count=1" % (d, str(fileNum), str(fileSize))
                os.system(createFileCommand)
                fileNum += 1


    def periodicWakeup(self):
        for m in self.machines.values():
            pa = PingAgent(m.hostname, self.status, m.machineName)
            pa.start()
        knownPeers = ""
        for m in self.machines.values():
            if len(m.hostname) > 1 and len(m.port) > 1:
                knownPeers += (" " + str(m.hostname))
                knownPeers += (" " + str(m.port))
        for m in self.machines:
            self.periodicWakeupForMachine(self.machines[m], self.machinesObjects[m], knownPeers)
        self.window.show_all()

    def periodicWakeupForMachine(self, machine, machineObject, knownPeers):
        if not self.status.has_key(machine.machineName):
            self.setStatusUnknown(machineObject)
            return
        if self.status[machine.machineName] == "UNREACHABLE":
            self.setStatusUnreachable(machineObject)
        if self.status[machine.machineName] == "OK":
            proxy = self.machinesObjects[machine.machineName].proxy
            assert proxy is not None
            try:
                if (proxy.isWorking() == 1):
                    self.setStatusWorking(machineObject)
                else:
                    self.setStatusNotWorking(machineObject)
                    if machine.daemonized:
                        self.start_machine(machine.machineName, knownPeers)
            except Exception as e:
                self.setStatusNotWorking(machineObject)
                if machine.daemonized:
                    self.start_machine(machine.machineName, knownPeers)

    def setStatusWorking(self, machineObject):
        machineObject.statusImage.set_from_file("images/greenLamp.png")

    def setStatusNotWorking(self, machineObject):
        machineObject.statusImage.set_from_file("images/blueLamp.png")

    def setStatusUnreachable(self, machineObject):
        machineObject.statusImage.set_from_file("images/redLamp.png")

    def setStatusUnknown(self, machineObject):
        machineObject.statusImage.set_from_file("images/greyLamp.png")

class p2pBackupAdministrationGui:
    def __init__(self, title):
        self.machines = shelve.open(configurationFileName, writeback=True) # this is dictionary from machine_name to Machine
        self.selectedMachines = []
        self.machinesObjects = {}
        gtk.gdk.threads_init()
        self.window = gtk.Window()
        self.machinesMonitor = MachinesMonitor(self.machines, self.machinesObjects, self.window)
        thread.start_new_thread(self.machinesMonitor.run,())
        self.title = title
        self.window.set_title(title)
        #self.window.set_size_request(600, -1)
        self.window.connect("destroy", self.destroy)
        self.window.set_resizable(False)
        self.create_interior()
        self.window.show_all()
        
    def create_interior(self):
        self.mainbox=gtk.HBox()
        self.table = gtk.Table(4, 6)
        self.window.add(self.mainbox)
        # create a notebook
        self.notebook = gtk.Notebook()
        # position the page indicators
        self.notebook.set_tab_pos(gtk.POS_TOP)
        self.table.attach(self.notebook, 0, 6, 0, 1)
        self.notebook.show()
        self.table.show()
        self.notebook.set_show_tabs(True)
        self.status_bar = gtk.Statusbar()
        self.context_id = self.status_bar.get_context_id("Statusbar")
        self.leftBoxWidth = 550
        self.leftCheckBoxWidth = 330

        self.machineList = gtk.VBox()
        scrolledmachinesList = gtk.ScrolledWindow()
        scrolledmachinesList.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledmachinesList.add_with_viewport(self.machineList)
        scrolledmachinesList.show()
        scrolledmachinesList.set_size_request(self.leftBoxWidth, 350)
        lowBottomsBox = gtk.HBox()

        button_add_machine = gtk.Button("Add machine")
        button_add_machine.connect("clicked", self.add_machine_clicked, self.context_id)
        button_add_machine.set_size_request(100, 40)

        lowBottomsBox.pack_start(button_add_machine)
        lowBottomsBox.pack_start(gtk.VSeparator())

        button_select_all = gtk.Button("Select all")
        button_select_all.connect("clicked", self.select_all_machines, self.context_id)
        button_select_all.set_size_request(100, 40)

        lowBottomsBox.pack_start(button_select_all)
        lowBottomsBox.pack_start(gtk.VSeparator())

        button_deselect_all = gtk.Button("Deselect all")
        button_deselect_all.connect("clicked", self.deselect_all_machines, self.context_id)
        button_deselect_all.set_size_request(100, 40)

        lowBottomsBox.pack_start(button_deselect_all)

        leftVBox = gtk.VBox()
        leftVBox.set_size_request(self.leftBoxWidth, -1)
        leftVBox.pack_start(scrolledmachinesList)
        leftVBox.pack_start(gtk.HSeparator())
        leftVBox.pack_end(lowBottomsBox)    

        # add or remove the notebook page tabs
        frame = gtk.Frame("Administration menu")
        hBox = gtk.HBox(True)
        vBox = gtk.VBox(True)
        vBox.set_size_request(200, 200)

        button_start = gtk.Button("Start machines")
        button_start.connect("clicked", self.start_machines, self.context_id)
        button_start.set_size_request(200, 40)

        button_stop = gtk.Button("Stop machines")
        button_stop.connect("clicked", self.stop_machines, self.context_id)
        button_stop.set_size_request(200, 40)

        button_daemonize = gtk.Button("Daemonize machines")
        button_daemonize.connect("clicked", self.daemonize_machines, self.context_id)
        button_daemonize.set_size_request(200, 40)

        button_undaemonize = gtk.Button("Undaemonize machines")
        button_undaemonize.connect("clicked", self.undaemonize_machines, self.context_id)
        button_undaemonize.set_size_request(200, 40)

        vBox.pack_start(gtk.HSeparator())
        vBox.pack_start(button_start, False, False)
        vBox.pack_start(gtk.HSeparator())
        vBox.pack_start(button_stop, False, False)
        vBox.pack_start(gtk.HSeparator())
        vBox.pack_start(button_daemonize, False, False)
        vBox.pack_start(gtk.HSeparator())
        vBox.pack_start(button_undaemonize, False, False)
        vBox.pack_start(gtk.HSeparator())
        hBox.pack_start(vBox, False, False)
        frame.add(hBox)
        label = gtk.Label("Administration menu")        
        frame.show()
        
        frame2 = gtk.Frame("Reports Menu")
        label2 = gtk.Label("Reports Menu")
        hBox_frame2 = gtk.HBox(True)
        vBox_frame2 = gtk.VBox(True)
        vBox_frame2.set_size_request(200, 200)

        button_print_placement = gtk.Button("Replica Placement")
        button_print_placement.connect("clicked", self.print_placement, self.context_id)
        button_print_placement.set_size_request(200, 40)

        vBox_frame2.pack_start(button_print_placement, False, False)
        vBox_frame2.pack_start(gtk.HSeparator())
        hBox_frame2.pack_start(vBox_frame2, False, False)
        frame2.add(hBox_frame2)

        frame3 = gtk.Frame("Test Menu")
        label3 = gtk.Label("Test Menu")
        hbox_create_test_data = gtk.HBox()
        testDataSizeLabel = gtk.Label("Data size (in KB): ")
        testDataSizeLabel.set_size_request(200, 50)
        testDataSizeEntry = gtk.Entry(max=0)
        testDataSizeEntry.set_size_request(150, 30)
        hbox_create_test_data.pack_start(testDataSizeLabel)
        hbox_create_test_data.pack_end(testDataSizeEntry)

        hBox_frame3 = gtk.HBox(True)
        vBox_frame3 = gtk.VBox(True)
        vBox_frame3.set_size_request(200, 200)

        button_create_data = gtk.Button("Create initial data")
        button_create_data.connect("clicked", self.create_initial_data, testDataSizeEntry)
        button_create_data.set_size_request(200, 40)

        button_mark_backup = gtk.Button("Mark for backup")
        button_mark_backup.connect("clicked", self.mark_for_backup, self.context_id)
        button_mark_backup.set_size_request(200, 40)

        button_delete_data = gtk.Button("Delete all data")
        button_delete_data.connect("clicked", self.delete_all_data, self.context_id)
        button_delete_data.set_size_request(200, 40)

        vBox_frame3.pack_start(hbox_create_test_data, False, False)
        vBox_frame3.pack_start(button_create_data, False, False)
        vBox_frame3.pack_start(gtk.HSeparator())
        vBox_frame3.pack_start(button_mark_backup, False, False)
        vBox_frame3.pack_start(gtk.HSeparator())
        vBox_frame3.pack_start(button_delete_data, False, False)
        vBox_frame3.pack_start(gtk.HSeparator())
        hBox_frame3.pack_start(vBox_frame3, False, False)
        frame3.add(hBox_frame3)

        self.notebook.prepend_page(frame3, label3)
        self.notebook.prepend_page(frame2, label2)
        self.notebook.append_page(frame, label)
 
        self.mainbox.pack_start(leftVBox)       
        self.mainbox.pack_end(self.table)

        for m in self.machines.values():
            self.add_machine_checkbox(m)

    def select_all_machines(self, widget, d):
        for key in self.machines.keys():
            self.machinesObjects[key].checkbox.set_active(True)

    def deselect_all_machines(self, widget, d):
        for key in self.machines.keys():
            self.machinesObjects[key].checkbox.set_active(False)

    def machine_checkbox_toogled(self, widget, m):
        if widget.get_active():
            self.selectedMachines.append(m)
        else:
            self.selectedMachines.remove(m)

    def add_machine(self, m):
        self.machines[m.machineName] = m
        self.machines.sync()
        self.add_machine_checkbox(m)
        self.window.show_all()

    def edit_machine(self, button, machine, newMachine):
        button.set_label(newMachine.machineName)
        old_key = machine.machineName
        self.machines.pop(old_key)
        self.machines[newMachine.machineName] = newMachine
        machine.copy(newMachine)
        self.machinesObjects[newMachine.machineName] = self.machinesObjects.pop(old_key)
        self.machines.sync()
        self.machinesObjects[machine.machineName].proxy = \
            xmlrpclib.ServerProxy("http://" + str(machine.hostname) + ":" + str(machine.port), allow_none = True)

    def remove_machine_prompt(self, widget, hBox, hSeparator, button, m):
        question = gtk.MessageDialog(type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_YES_NO)
        question.set_markup("Do you want to permanently remove the machine from your database?")
        question.set_title("Warning")
        response = question.run()
        if response == gtk.RESPONSE_YES:
            question.destroy()
            self.remove_machine(hBox, hSeparator, button, m)
        else:
            question.destroy()

    def remove_machine(self, hBox, hSeparator, button, m):
        del self.machines[m.machineName]
        self.machines.sync()
        self.machineList.remove(hBox)
        self.machineList.remove(hSeparator)
        del self.machinesObjects[m.machineName]
        self.window.show_all()

    def add_machine_checkbox(self, m):
        hBox = gtk.HBox()
        hSeparator = gtk.HSeparator()

        button = gtk.CheckButton(m.machineName)
        button.set_size_request(self.leftCheckBoxWidth, 40)
        button.connect("toggled", self.machine_checkbox_toogled, m)
        self.machinesObjects[m.machineName] = MachineObjects()
        self.machinesObjects[m.machineName].proxy = xmlrpclib.ServerProxy("http://" + str(m.hostname) + ":" + str(m.port), allow_none = True)
        self.machinesObjects[m.machineName].checkbox = button

        statusImage = gtk.Image()
        self.machinesObjects[m.machineName].statusImage = statusImage
        statusImage.set_size_request(10, 10)
        statusImage.set_from_file("images/greyLamp.png")

        daemonImage = gtk.Image()
        self.machinesObjects[m.machineName].daemonImage = daemonImage
        daemonImage.set_size_request(10, 10)
        if m.daemonized:
            daemonImage.set_from_file("images/daemon_tools.png")
        else:
            daemonImage.set_from_file("images/non_daemon_tools.png")

        vBox = gtk.VBox()
        vBox.set_size_request(30, -1)
        button_remove = gtk.Button("Remove")
        button_remove.connect("clicked", self.remove_machine_prompt, hBox, hSeparator, button, m)

        button_edit = gtk.Button("Edit")
        button_edit.connect("clicked", self.edit_machine_clicked, button, m)

        vBox.pack_start(button_remove)
        vBox.pack_start(button_edit)

        hBox.pack_start(daemonImage)
        hBox.pack_start(button)
        hBox.pack_start(statusImage)
        hBox.pack_start(vBox)

        self.machineList.pack_start(hBox)
        self.machineList.pack_start(hSeparator)

    #def select_files(self, w, data):
    #    print "Selected files"
    #    fileSelector = FileSelector()
    #    fileSelector.show()

    def create_initial_data(self, w, entry):
        size = entry.get_text()
        try:
            size = int(size)
        except:
            print "The given size is not a number"
            return
        for machine in self.selectedMachines:
            self.machinesMonitor.create_initial_data(machine.machineName, size)

    def print_placement(self, w, data):
        for machine in self.selectedMachines:
            self.machinesMonitor.print_placement(machine.machineName)

    def start_machines(self, w, data):
        knownPeers = ""
        for m in self.machines.values():
            if len(m.hostname) > 1 and len(m.port) > 1:
                knownPeers += (" " + str(m.hostname))
                knownPeers += (" " + str(m.port))
        for machine in self.selectedMachines:
            self.machinesMonitor.start_machine(machine.machineName, knownPeers)

    def stop_machines(self, w, data):
        for machine in self.selectedMachines:
            self.machinesMonitor.stop_machine(machine.machineName)

    def daemonize_machines(self, w, data):
        for machine in self.selectedMachines:
            machine.daemonized = True
            self.machinesObjects[machine.machineName].daemonImage.set_from_file("images/daemon_tools.png")
            self.machines[machine.machineName] = machine
        self.machines.sync()
        self.window.show_all()

    def undaemonize_machines(self, w, data):
        for machine in self.selectedMachines:
            machine.daemonized = False
            self.machinesObjects[machine.machineName].daemonImage.set_from_file("images/non_daemon_tools.png")
            self.machines[machine.machineName] = machine
        self.machines.sync()
        self.window.show_all()

    def mark_for_backup(self, w, data):
        for machine in self.selectedMachines:
            self.machinesMonitor.mark_for_backup(machine.machineName)

    def delete_all_data(self, w, data):
        for machine in self.selectedMachines:
            self.machinesMonitor.delete_all_data(machine.machineName)

    def add_machine_clicked(self, w, data):
        self.editMachineWindow(self.add_machine, Machine())

    def edit_machine_clicked(self, widget, button, m):
        self.editMachineWindow(lambda x: self.edit_machine(button, m, x), m)

    def editMachineWindow(self, functionOnSave, machine):
        window = gtk.Window()
        window.set_title("Edit machine")
        vbox = gtk.VBox()
        scrolledView = gtk.ScrolledWindow()
        scrolledView.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledView.add_with_viewport(vbox)
        scrolledView.show()
        scrolledView.set_size_request(700, 600)

        window.add(scrolledView)
        hbox1 = gtk.HBox()
        nameLabel =                     gtk.Label("Machine label:                    ")
        hostnameLabel =                 gtk.Label("Hostname:                          ")
        portLabel =                     gtk.Label("Port number:                       ")
        usernameLabel =                 gtk.Label("Username:                          ")
        homeDirLabel =                  gtk.Label("Home directory:                  ")
        storageDirLabel =               gtk.Label("Local storage directory:      ")
        directoriesForBackupLabel =     gtk.Label("Directories for backup:       ")
        mainPathLabel =                 gtk.Label("Main application path:         ")
        totalIncommingBwBoundLabel =    gtk.Label("Incomming bandwidth (B/s):")
        desiredBackWindowLabel =        gtk.Label("Backup window (sec):          ")
        totalSpaceLabel =               gtk.Label("Total space (B):                    ")

        emptyLabel1 = gtk.Label("")
        emptyLabel2 = gtk.Label("")
        entriesWidth = 400

        nameLabel.set_size_request(200, 50)
        nameEntry = gtk.Entry(max=0)
        nameEntry.set_text(machine.machineName)
        nameEntry.set_size_request(entriesWidth, 30)
        hbox1.pack_start(nameLabel)
        hbox1.pack_end(nameEntry)

        hbox2 = gtk.HBox()
        hostnameLabel.set_size_request(200, 50)
        hostnameEntry = gtk.Entry(max=0)
        hostnameEntry.set_text(machine.hostname)
        hostnameEntry.set_size_request(entriesWidth, 30)
        hbox2.pack_start(hostnameLabel)
        hbox2.pack_end(hostnameEntry)

        hbox3 = gtk.HBox()
        portLabel.set_size_request(200, 50)
        portEntry = gtk.Entry(max=0)
        portEntry.set_text(machine.port)
        portEntry.set_size_request(entriesWidth, 30)
        hbox3.pack_start(portLabel)
        hbox3.pack_end(portEntry)

        hbox4 = gtk.HBox()
        usernameLabel.set_size_request(200, 50)
        usernameEntry = gtk.Entry(max=0)
        usernameEntry.set_text(machine.username)
        usernameEntry.set_size_request(entriesWidth, 30)
        hbox4.pack_start(usernameLabel)
        hbox4.pack_end(usernameEntry)

        hbox5 = gtk.HBox()
        homeDirLabel.set_size_request(200, 50)
        homeDirEntry = gtk.Entry(max=0)
        homeDirEntry.set_text(machine.homeDir)
        homeDirEntry.set_size_request(entriesWidth, 30)
        hbox5.pack_start(homeDirLabel)
        hbox5.pack_end(homeDirEntry)

        hbox10 = gtk.HBox()
        storageDirLabel.set_size_request(200, 50)
        storageDirEntry = gtk.Entry(max=0)
        storageDirEntry.set_text(machine.storageDir)
        storageDirEntry.set_size_request(entriesWidth, 30)
        hbox10.pack_start(storageDirLabel)
        hbox10.pack_end(storageDirEntry)

        hbox11 = gtk.HBox()
        mainPathLabel.set_size_request(200, 50)
        mainPathEntry = gtk.Entry(max=0)
        mainPathEntry.set_text(machine.mainPath)
        mainPathEntry.set_size_request(entriesWidth, 30)
        hbox11.pack_start(mainPathLabel)
        hbox11.pack_end(mainPathEntry)

        hbox12 = gtk.HBox()
        totalIncommingBwBoundLabel.set_size_request(200, 50)
        totalIncommingBwBoundEntry = gtk.Entry(max=0)
        totalIncommingBwBoundEntry.set_text(machine.totalIncommingBwBound)
        totalIncommingBwBoundEntry.set_size_request(entriesWidth, 30)
        hbox12.pack_start(totalIncommingBwBoundLabel)
        hbox12.pack_end(totalIncommingBwBoundEntry)

        hbox13 = gtk.HBox()
        desiredBackWindowLabel.set_size_request(200, 50)
        desiredBackWindowEntry = gtk.Entry(max=0)
        desiredBackWindowEntry.set_text(machine.desiredBackWindow)
        desiredBackWindowEntry.set_size_request(entriesWidth, 30)
        hbox13.pack_start(desiredBackWindowLabel)
        hbox13.pack_end(desiredBackWindowEntry)

        hbox14 = gtk.HBox()
        totalSpaceLabel.set_size_request(200, 50)
        totalSpaceEntry = gtk.Entry(max=0)
        totalSpaceEntry.set_text(machine.totalSpace)
        totalSpaceEntry.set_size_request(entriesWidth, 30)
        hbox14.pack_start(totalSpaceLabel)
        hbox14.pack_end(totalSpaceEntry)

        hbox8 = gtk.HBox()
        directoriesForBackupLabel.set_size_request(200, 50)
        dataToBackupStore = gtk.ListStore(str)
        for val in machine.directoriesForBackup:
            dataToBackupStore.append([val])

        treeView = gtk.TreeView(dataToBackupStore)
        treeView.set_rules_hint(True)
        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Path", rendererText, text=0)    
        treeView.append_column(column)
        scrolledDataToBackupBox = gtk.ScrolledWindow()
        scrolledDataToBackupBox.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledDataToBackupBox.add_with_viewport(treeView)
        scrolledDataToBackupBox.show()
        scrolledDataToBackupBox.set_size_request(entriesWidth, 200)
        hbox8.pack_start(directoriesForBackupLabel)
        hbox8.pack_end(scrolledDataToBackupBox)

        hbox9 = gtk.HBox()
        emptyLabel2.set_size_request(200, 50)
        newDirForBackupEntry = gtk.Entry(max=0)
        newDirForBackupEntry.set_size_request(entriesWidth, 30)
        hbox9.pack_start(emptyLabel2)
        hbox9.pack_end(newDirForBackupEntry)
        hbox9.set_size_request(400, 30)

        dirsForBackup = machine.directoriesForBackup
        hbox6 = gtk.HBox()
        emptyLabel1.set_size_request(200, 50)
        button_add_dir = gtk.Button("Add new directory")
        button_add_dir.connect("clicked",
                               self.added_new_backup_dir,
                               dirsForBackup,
                               newDirForBackupEntry,
                               dataToBackupStore,
                               window
        )
        button_remove_dir = gtk.Button("Remove selected directory")
        button_remove_dir.connect("clicked",
                               self.remove_selected_backup_dir,
                               dirsForBackup,
                               treeView,
                               dataToBackupStore,
                               window
        )
        hbox6_1 = gtk.HBox(True)
        hbox6_1.set_size_request(entriesWidth, -1)
        hbox6_1.pack_start(button_add_dir)
        hbox6_1.pack_end(button_remove_dir)
        hbox6.pack_start(emptyLabel1)
        hbox6.pack_end(hbox6_1)
        hbox6.set_size_request(400, 30)

        hbox7 = gtk.HBox(True)
        button_save = gtk.Button("save")
        button_save.connect("clicked",
                            self.extractMachine,
                            functionOnSave,
                            nameEntry,
                            hostnameEntry,
                            portEntry,
                            usernameEntry,
                            homeDirEntry,
                            storageDirEntry,
                            mainPathEntry,
                            totalIncommingBwBoundEntry,
                            desiredBackWindowEntry,
                            totalSpaceEntry,
                            dirsForBackup,
                            window
        )
        button_save.set_size_request(200, 30)
        hbox7.pack_start(gtk.Label(""))
        hbox7.pack_start(button_save)
        hbox7.pack_end(gtk.Label(""))
        hbox7.set_size_request(400, 30)

        vbox.pack_start(hbox1)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox2)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox3)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox4)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox5)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox10)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox11)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox12)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox13)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox14)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(hbox8)
        vbox.pack_start(hbox9)
        vbox.pack_start(hbox6)
        vbox.pack_start(gtk.HSeparator())
        vbox.pack_start(gtk.Label(""))
        vbox.pack_start(hbox7)
        window.show_all()

    def extractMachine(self,
                       widget,
                       callFun,
                       nameEntry,
                       hostnameEntry,
                       portEntry,
                       usernameEntry,
                       homeDirEntry,
                       storageDirEntry,
                       mainPathEntry,
                       totalIncommingBwBoundEntry,
                       desiredBackWindowEntry,
                       totalSpaceEntry,
                       dirsForBackup,
                       window
        ):
        m = Machine()
        m.machineName = nameEntry.get_text()
        m.hostname = hostnameEntry.get_text()
        m.port = portEntry.get_text()
        m.username = usernameEntry.get_text()
        m.homeDir = homeDirEntry.get_text()
        m.storageDir = storageDirEntry.get_text()
        m.mainPath = mainPathEntry.get_text()
        m.totalIncommingBwBound = totalIncommingBwBoundEntry.get_text()
        m.desiredBackWindow = desiredBackWindowEntry.get_text()
        m.totalSpace = totalSpaceEntry.get_text()
        m.directoriesForBackup = dirsForBackup
        window.destroy()
        callFun(m)

    def added_new_backup_dir(self, widget, dirList, newDirForBackupEntry, dataToBackupStore, window):
        dirName = newDirForBackupEntry.get_text()
        if len(dirName) > 0:
            dirList.append(dirName)
            dataToBackupStore.append([dirName])
            window.show_all()

    def remove_selected_backup_dir(self, widget, dirList, treeView, dataToBackupStore, window):
        selected_rows = treeView.get_selection().get_selected_rows()[1]
        if len(selected_rows) > 0:
            (selection,) = selected_rows[0]
            del dataToBackupStore[selection]
            del dirList[selection]
            window.show_all()

    def main(self):
        gtk.main()
  
    def destroy(self, w):
        gtk.main_quit()
        

m = p2pBackupAdministrationGui("p2pBackup")
m.main()

