#! /usr/bin/env python

import shelve
import time
import math
import thread
import config
import traceback
import copy
import os
import re
import sys
import xmlrpclib
from os import path
import socket
from datetime import datetime
from common import day_difference
from common import remoteCommand
from threading import Thread
from persistentDict import PersistentDict

configurationFileName = "configuration-persistent-data"
eventsFileName = "events-persistent-data"
filesToBackupFileName = "filesToBackup-persistent-data"
totalIncommingSizeFileName = "totalIncommingSize-persistent-data"
ttlsFileName = "ttls-persistent-data"
updateWaitingTimeFileName = "updateWaitingTime-persistent-data"
periodicEventsFileName = "periodicEvents-persistent-data"
toLeaveFileName = "toLeave-persistent-data"

machinesDict = None
machinesProxiesDict = None
eventsDict = None
periodicEventsDict = None
filesToBackupDict = None
totalIncommingSizeDict = None
ttlsDict = None
updateWaitingTimeDict = None

def tidyUp():
    machinesDict.close()
    machinesProxiesDict.close()
    eventsDict.close()
    periodicEventsDict.close()
    filesToBackupDict.close()
    totalIncommingSizeDict.close()
    ttlsDict.close()
    updateWaitingTimeDict.close()

if config.enviroment == "home":
    periodicMonitorIntervalSec = 20
    dataChunkSize = 10 * 1024
    totalIncommingSizeIntervalSec = 60
    ttlsGatherIntervalSec = 60
else:
    periodicMonitorIntervalSec = 5 * 60
    dataChunkSize = 50 * 1024 * 1024
    totalIncommingSizeIntervalSec = 15 * 60
    ttlsGatherIntervalSec = 15 * 60


socket.setdefaulttimeout(10)

def getAvgVar(durations):
    s = 0.0
    for d in durations:
        s += d
    av = s / len(durations)
    dev = 0.0
    for d in durations:
        dev += (av - d) ** 2
    dev = math.sqrt(dev / len(durations))
    return (av, dev)

def clean():
    os("rm -rf " + configurationFileName)
    os("rm -rf " + eventsFileName)
    os("rm -rf " + filesToBackupFileName)
    os("rm -rf " + totalIncommingSizeFileName)
    os("rm -rf " + updateWaitingTimeFileName)

class Machine:
    def __init__(self):
        self.machineName = ""
        self.hostname = ""
        self.port = ""
        self.username = ""
        self.homeDir = ""
        self.storageDir = ""
        self.daemonized = False
        self.directoriesForBackup = []
        self.mainPath = ""
        self.totalIncommingBwBound = 0
        self.desiredBackWindow = 0
        self.myTotalSpace = 0
        self.localDataSize = 0
        self.metadataPath = ""

class PingAgent(Thread):
    def __init__(self, user, host, dic, index, nr):
        Thread.__init__(self)
        self.user = user
        self.host = host
        self.dic = dic
        self.index = index
        self.nr = nr

    def run(self):
        try:
            toLeave = shelve.open(toLeaveFileName + "-" + str(self.nr), writeback=True)
            if toLeave.has_key(self.index) and (toLeave[self.index] == True):
                self.dic[self.index] = "LEAVE"
                return
            toLeave.close()

            output = os.popen("ssh -f -o ConnectTimeout=30 " + self.user + "@" + self.host + " pwd", "r")
            line = output.readline()
            if (len(line) < 2) or ("timed out" in line):
                self.dic[self.index] = "LEAVE"
            else:
                self.dic[self.index] = "ENTER"
        except KeyboardInterrupt:
            tidyUp()
            sys.exit(1)
        except:
            pass
        #out = os.popen('ping -c 3 ' + str(self.host))
        #output = out.read()
        #rttS = re.search('time=([0-9]*)', output)
        #ttlS = re.search('ttl=([0-9]*)', output)
        #if rttS != None and ttlS != None:
        #    self.dic[self.index] = "ENTER"
        #else:
        #    self.dic[self.index] = "LEAVE"

def uploadCodeForMachine(m):
    createDirectoryCommand = "ssh -f -o ConnectTimeout=2 " + m.username + "@" + m.hostname + " mkdir -p %s" % m.mainPath
    copyCommand = "scp " + os.getcwd() + "/*.py" + " " +  m.username + "@" + m.hostname + ":" + m.mainPath
    copyCommand2 = "scp " + os.getcwd() + "/*.pub" + " " +  m.username + "@" + m.hostname + ":" + m.mainPath
    print createDirectoryCommand
    print copyCommand
    os.system(createDirectoryCommand)
    os.system(copyCommand)
    os.system(copyCommand2)

def printStatus(status, events):
    current = time.time()
    for k in status.keys():
        print str(datetime.today()) + " MACHINES-MONITOR: Printing status for machine: " + str(k)
        if events.has_key(k):
            peerEvents = events[k]
            oldStatus, t = peerEvents[-1]
            if oldStatus != status[k]:
                peerEvents.append((status[k], current))
            events[k] = peerEvents
        else:
            peerEvents = [(status[k], current)]
            events[k] = peerEvents
    events.sync()

def gather_incomming_size(k, machinesProxies, totalIncommingSize):
    print str(datetime.today()) + " MACHINES-MONITOR: Gathering totalIncommingSize for machine: " + str(k)
    if totalIncommingSize.has_key(k):
        e = totalIncommingSize[k]
        last_e = e[-1]
        last_size, t = last_e
    else:
        e = []
        t = 0
    if time.time() - t > totalIncommingSizeIntervalSec:
        print str(datetime.today()) + " MACHINES-MONITOR: Time to gather totalIncommingSize for machine: " + str(k)
        try:
            proxy = machinesProxies[k]
            e.append((float(proxy.getTotalIncommingSize()), time.time()))
            totalIncommingSize[k] = e
        except KeyboardInterrupt:
            tidyUp()
            sys.exit(1)
        except Exception as ex:
            traceback.print_exc()
            print ex
            pass

def gather_ttls(k, machinesProxies, ttls):
    if ttls.has_key(k):
        e = ttls[k]
        vals, t = e
    else:
        t = 0
    if time.time() - t > ttlsGatherIntervalSec:
        try:
            proxy = machinesProxies[k]
            vals = proxy.getTtls()
            ttls[k] = (vals, time.time())
        except KeyboardInterrupt:
            tidyUp()
            sys.exit(1)
        except Exception as ex:
            #traceback.print_exc()
            #print ex
            pass

def print_ttls(nr):
    try:
        result = {}
        result['far'] = {}
        far = []
        result['close'] = {}
        close = []
        ttls = shelve.open(ttlsFileName + "-" + str(nr), writeback=True)
        for k in ttls.keys():
            vals, t = ttls[k]
            for name in vals.keys():
                for v in vals[name].keys():
                    try:
                        result[name][v] += vals[name][v]
                    except KeyError:
                        result[name][v] = vals[name][v] 

        for name in result.keys():
            print str(name)
            for v in result[name].keys():
                if name == 'close':
                    for i in range(0, int(result[name][v])):
                        close.append(float(v))
                else:
                    for i in range(0, int(result[name][v])):
                        far.append(float(v))
                print "    " + str(v) + " <- " + str(result[name][v])
        print "Far summary:"
        av, dev = getAvgVar(far)
        print "Average and standard deviations: " + str(av) + ", " + str(dev)
        print "Close summary:"
        av, dev = getAvgVar(close)
        print "Average and standard deviations: " + str(av) + ", " + str(dev)
        print "Summary:"
        av, dev = getAvgVar(far + close)
        print "Average and standard deviations: " + str(av) + ", " + str(dev)
    except KeyboardInterrupt:
        ttls.close()
        sys.exit(1)
            

def makeBackup(k, machines, machinesProxies, filesToBackup):
    print str(datetime.today()) + " MACHINES-MONITOR: Making backup, machine: " + str(k)
    proxy = machinesProxies[k]
    m = machines[k]
    l = filesToBackup[k]
    try:
        proxy.makeBackupAsync(l)
    except KeyboardInterrupt:
        tidyUp()
        sys.exit(1)
    except Exception as ex:
        traceback.print_exc()
        print ex

def start_machines(status, machines, machinesProxies, periodicEvents, filesToBackup, updateWaitingTime, otherIps, nr):
    global dataChunkSize
    for m in machines.values():
        start_single_machine(m, status, machines, machinesProxies, periodicEvents, filesToBackup, updateWaitingTime, otherIps, nr)
        #thread.start_new_thread(
        #    start_single_machine,
        #    (m, status, machines, machinesProxies, periodicEvents, filesToBackup, updateWaitingTime, otherIps)
        #)

def start_single_machine(m, status, machines, machinesProxies, periodicEvents, filesToBackup, updateWaitingTime, otherIps, nr):
    print str(datetime.today()) + " MACHINES-MONITOR: Considering machine: " + str(m.machineName)
    sys.stdout.flush()
    try:
        k = m.machineName
        proxy = machinesProxies[k]
        periodicEvent = True
        if periodicEvents.has_key(k):
            t = periodicEvents[k]
            # some break
            if day_difference(t, time.time()) <= 0:
                periodicEvent = False


        if status.has_key(k) and (status[k] != "ENTER"):
            print str(datetime.today()) + " MACHINES-MONITOR: Machine unavailable: " + str(m.machineName)
            return

        print str(datetime.today()) + " MACHINES-MONITOR: Periodic event for machine: " + str(m.machineName)
        try:
            proxy.isWorking()
            print str(datetime.today()) + " MACHINES-MONITOR: Machine is working: " + str(m.machineName)
        except KeyboardInterrupt:
            tidyUp()
            sys.exit(1)
        except:
            toLeave = shelve.open(toLeaveFileName + "-" + str(nr))
            if toLeave.has_key(k) and (toLeave[k] == True):
                print str(datetime.today()) + " MACHINES-MONITOR: Killing machine: " + str(m.machineName)
                command = "pkill -U " + m.username + " -f .*python.*"
                remoteCommand(m.username, m.hostname, "~", command)
            else:
                #reconstruct others data
                if config.enviroment == "planetLab":
                    uploadCodeForMachine(m)
                command = "python -u reconstructData.py " + \
                     str(int(m.port) + 1) + " " + m.storageDir  + " " + str(dataChunkSize) + " " + m.homeDir + " " + m.hostname + " " + \
                     m.metadataPath
                remoteCommand(m.username, m.hostname, m.mainPath, command)
                print str(datetime.today()) + " MACHINES-MONITOR: Starting machine: " + str(m.machineName)
                command = "python -u mainFacade.py " +\
                    m.hostname + " " + str(m.port) + " " + m.homeDir + " " + m.storageDir + " " + m.homeDir + "/statistics.txt" + " " +\
                    str(m.totalIncommingBwBound) + " " + str(m.desiredBackWindow) + " " + str(m.myTotalSpace) + " " +\
                    str(m.metadataPath) + " " + str(otherIps)
                remoteCommand(m.username, m.hostname, m.mainPath, command)
            toLeave.close()

        if periodicEvent:
            for i in range(0, 10):
                if i == 9:
                    raise ZeroDivisionError()
                try:
                    proxy.isWorking()
                    break
                except ZeroDivisionError:
                    time.sleep(1)
            #update files
            makeBackup(k, machines, machinesProxies, filesToBackup)
            #update periodic events
            periodicEvents[k] = time.time()
            periodicEvents.sync()
            updateWaitingTime.sync()
    except KeyboardInterrupt:
        tidyUp()
        sys.exit(1)
    except Exception as ex:
        #traceback.print_exc()
        #print ex
        pass   

def gather_update_waiting_time(k, machinesProxies, updateWaitingTime):
    proxy = machinesProxies[k]
    try:
        ownerId, vers = proxy.getFileVersions()
        for f in vers.keys():
            if not updateWaitingTime.has_key([ownerId, f]):
                updateWaitingTime.set([ownerId, f], [(vers[f], time.time(), [])])
            else:
                l = updateWaitingTime.get([ownerId, f])
                l_last = l[-1]
                v, t, l2 = l_last
                if vers[f] > v:
                    l.append((vers[f], time.time(), []))
                    updateWaitingTime.set([ownerId, f], l)

        vers = proxy.getOthersFilesVersions()
        for ownerId in vers.keys():
            for f in vers[ownerId].keys():
                if updateWaitingTime.has_key([ownerId, f]):
                    l = updateWaitingTime.get([ownerId, f])
                    index = -1
                    while -1 * index <= len(l):
                        last_e = l[index]
                        v, t, reps = last_e
                        if (len(reps) >= 3):
                            break
                        if vers[ownerId][f] < v:
                            index -= 1
                            continue
                        else:
                            alreadyAck = False
                            for r in reps:
                                r_n, r_t = r
                                if r_n == k:
                                    alreadyAck = True
                            if not alreadyAck:
                                reps.append((k, time.time() - t))
                            l[index] = (v, t, reps)
                            index -= 1
                    updateWaitingTime.set([ownerId, f], l)
        updateWaitingTime.sync()
    except KeyboardInterrupt:
        tidyUp()
        sys.exit(1)
    except Exception as ex:
        #traceback.print_exc()
        #print ex
        pass

    
def create_single_file_from_desc(k, machines, filesToBackup):
    m = machines[k]
    l = filesToBackup[k]
    for e in l:
        filename, size = e
        d = os.path.dirname(filename)
        createDirectoryCommand = "ssh -f -o ConnectTimeout=2 " + m.username + "@" + m.hostname + " mkdir -p %s" % d
        ret = os.system(createDirectoryCommand)
        createFileCommand = "ssh -f -o ConnectTimeout=2 " + m.username + "@" + m.hostname + \
             " dd if=/dev/zero of=%s bs=%s count=1" % (filename, str(size))
        ret = os.system(createFileCommand)


status = {}
def periodicWakeup(machines, machinesProxies, events, periodicEvents, filesToBackup, totalIncommingSize, ttls, updateWaitingTime, allIps, nr):
    print str(datetime.today()) + " MACHINES-MONITOR: Periodic wakeup"
    start_machines(status, machines, machinesProxies, periodicEvents, filesToBackup, updateWaitingTime, allIps, nr)
    for k in status.keys():
        if status[k] == "ENTER":
            try:
                gather_incomming_size(k, machinesProxies, totalIncommingSize)
                gather_update_waiting_time(k, machinesProxies, updateWaitingTime)
                gather_ttls(k, machinesProxies, ttls)
            except KeyboardInterrupt:
                tidyUp()
                sys.exit(1)
            except Exception as e:
                logs.trace(e)
    ttls.sync()
    totalIncommingSize.sync()
    time.sleep(periodicMonitorIntervalSec)


def periodicMonitor(machines, events, nr):
    global status
    while True:
        print str(datetime.today()) + " MACHINES-MONITOR: Periodic monitor"
        for m in machines.values():
            try:
                pa = PingAgent(m.username, m.hostname, status, m.machineName, nr)
                pa.start()
            except KeyboardInterrupt:
                tidyUp()
                sys.exit(1)
            except:
                pass
        time.sleep(periodicMonitorIntervalSec)
        printStatus(status, events)
        

def print_events(nr):
    events = shelve.open(eventsFileName + "-" + str(nr), writeback=True)
    for k in events.keys():
        print str(k)
        e = events[k]
        for single_e in e:
            e_name, t = single_e
            print "     " + str(e_name) + ", " + str(datetime.fromtimestamp(t)) 

def print_report(nr):
    try:
        machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
        machinesProxies = {}
        for m in machines.values():
            machinesProxies[m.machineName] = xmlrpclib.ServerProxy("http://" + str(m.hostname) + ":" + str(m.port), allow_none = True)
        for k in machinesProxies.keys():
            try:
                proxy = machinesProxies[k]
                res = proxy.getAvailability()
                print "Peer " + str(k) + ": availability = " + str(res)
            except KeyboardInterrupt:
                machines.close()
                sys.exit(1)
            except:
                print "Peer " + str(k) + " unreachable"
    except KeyboardInterrupt:
        machines.close()
        sys.exit(1)

def print_incomming_sizes(nr):
    try:
        sizes = shelve.open(totalIncommingSizeFileName + "-" + str(nr), writeback=True)
        for k in sizes.keys():
            print str(k)
            e = sizes[k]
            for single_e in e:
                s, t = single_e
                print "     " + str(s) + ", " + str(datetime.fromtimestamp(t))
    except KeyboardInterrupt:
        sizes.close()
        sys.exit(1)

def print_incomming_sizes2(nr):
    try:
        sizes = shelve.open(totalIncommingSizeFileName + "-" + str(nr), writeback=True)
        for k in sizes.keys():
            e = sizes[k]
            s, t = e[-1]
            if float(s) > 0:
            print str(k)
            print "     " + str(s) + ", " + str(datetime.fromtimestamp(t))
    except KeyboardInterrupt:
        sizes.close()
        sys.exit(1)

def print_update_waiting_time():
    try:
        notFinishedNum = 0
        oneReplicaNum = 0
        twoReplicaNum = 0
        threeReplicaNum = 0
        timeSum = 0
        timeNum = 0
        updateWaitingTime = PersistentDict(updateWaitingTimeFileName)
        for k in updateWaitingTime.keys([]):
            perUser = updateWaitingTime.get([k])
            for f in updateWaitingTime.keys([k]):
                try:
                    allVers = perUser[f]
                    for entr in allVers:
                        v, t, l = entr
                        if len(l) == 0:
                            #print str(k) + ", " + str(f) + ": " + str(entr)
                            notFinishedNum += 1
                        if len(l) == 1:
                            oneReplicaNum += 1
                        if len(l) == 2:
                            twoReplicaNum += 1
                        if len(l) >= 3:
                            threeReplicaNum += 1
                        for e in l:
                            k, tim = e
                            timeNum += 1
                            timeSum += tim
                except KeyboardInterrupt:
                    updateWaitingTime.close()
                    sys.exit(1)
                except Exception as e:
                    print "Error: " +  str(e)
                    pass
        print "Number of files not backed up: " + str(notFinishedNum)
        print "Number of files with one replica: " + str(oneReplicaNum)
        print "Number of files with two replicas: " + str(twoReplicaNum)
        print "Number of files with three replicas: " + str(threeReplicaNum)
        if timeNum > 0:
            print "Avg file backup time: " + str(timeSum / timeNum)
        updateWaitingTime.close()
    except KeyboardInterrupt:
        updateWaitingTime.close()
        sys.exit(1)

def copy_update_waiting_time():
    notFinishedNum = 0
    oneReplicaNum = 0
    twoReplicaNum = 0
    threeReplicaNum = 0
    timeSum = 0
    timeNum = 0
    updateWaitingTime = PersistentDict(updateWaitingTimeFileName)
    newUpdateWaitingTime = shelve.open(updateWaitingTimeFileName, writeback=True)
    for k in updateWaitingTime.keys([]):
        newUpdateWaitingTime[k] = {}
        perUser = updateWaitingTime.get([k])
        for f in updateWaitingTime.keys([k]):
            newUpdateWaitingTime[k][f] = perUser[f]
       
    newUpdateWaitingTime.sync()   
    newUpdateWaitingTime.close()

def create_files_desc(nr):
    global dataChunkSize
    print "Creating files desctriptions"
    machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
    filesToBackup = shelve.open(filesToBackupFileName + "-" + str(nr), writeback=True)

    for m in machines.values():
        size = m.localDataSize
        files = []
        for d in m.directoriesForBackup:
            remainSizeInDir = size / len(m.directoriesForBackup)
            fileNum = 0
            while remainSizeInDir > 0:
                fileSize = dataChunkSize # min(dataChunkSize, remainSizeInDir)
                remainSizeInDir -= fileSize
                filename = d + "/" + "file" + str(fileNum)
                files.append((filename, fileSize))
                fileNum += 1

        filesToBackup[m.machineName] = files
    filesToBackup.sync()

def upload_code(nr):
    machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)

    for m in machines.values():
        uploadCodeForMachine(m)


def printDesc(nr):
    filesToBackup = shelve.open(filesToBackupFileName + "-" + str(nr), writeback=True)
    for k in filesToBackup.keys():
        l = filesToBackup[k]
        print str(k)
        for e in l:
            filename, size = e
            print "    " + str(filename) + " <- " + str(size)

def create_files_from_desc(nr):
    print "Creating files from desctriptions"
    machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
    filesToBackup = shelve.open(filesToBackupFileName + "-" + str(nr), writeback=True)
    for k in filesToBackup.keys():
        create_single_file_from_desc(k, machines, filesToBackup)
        m = machines[k]
        l = filesToBackup[k]
        for e in l:
            filename, size = e
            d = os.path.dirname(filename)
            createDirectoryCommand = "ssh -f -o ConnectTimeout=2 " + m.username + "@" + m.hostname + " mkdir -p %s" % d
            os.system(createDirectoryCommand)
            createFileCommand = "ssh -f " + m.username + "@" + m.hostname + \
                 " dd if=/dev/zero of=%s bs=%s count=1" % (filename, str(size))
            os.system(createFileCommand)

def clear_data(m):
    command = "ssh -o ConnectTimeout=2 " + m.username + "@" + m.hostname + " rm -rf p2pBackup p2pBackupData " + m.metadataPath 
    for d in m.directoriesForBackup:
        command += (" " + str(d))

    os.system(command)

def leaveMachines(num, nr):
    machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
    allMachines = machines.keys()
    allMachines.sort()
    toLeave = shelve.open(toLeaveFileName + "-" + str(nr))
    i = 0
    for k in allMachines:
        m = machines[k]
        i += 1
        if (i > num):
            return
        toLeave[m.machineName] = True
        toLeave.sync()
        os.system ("ssh -o ConnectTimeout=3 " + m.username + "@" + m.hostname + " pkill -U " + m.username + " -f .*python.*")
    toLeave.close()

def get_logs(nr):
    machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
    for k in machines.keys():
        m = machines[k]
        logsPath = m.metadataPath + "/p2pBackupLogs-*"
        command = "scp " + m.username + "@" + m.hostname + ":" + logsPath + " ."
        os.system(command)

def kill_machines(nr):
    machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
    for k in machines.keys():
        m = machines[k]
        os.system ("ssh -o ConnectTimeout=3 " + m.username + "@" + m.hostname + " pkill -U " + m.username + " -f .*python.*")
        os.system ("ssh -o ConnectTimeout=3 " + m.username + "@" + m.hostname + " pkill -U " + m.username + " -f .*dd.if=/dev/zero.*")

def startMachines(num, nr):
    machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
    allMachines = machines.keys()
    allMachines.sort()
    toLeave = shelve.open(toLeaveFileName + "-" + str(nr))
    i = 0
    for k in allMachines:
        m = machines[k]
        i += 1
        if (i > num):
            return
        toLeave[m.machineName] = False
        toLeave.sync()
    toLeave.close()

if __name__ == '__main__':
    cmd = sys.argv[1]
    if cmd == "help":
        print "desc"
        print "files"
        print "clear"
        print "print_events"
        print "print_incomming_sizes"
        print "print_incomming_sizes2"
        print "print_desc"
        print "print_update_waiting_time"
        print "copy_update_waiting_time"
        print "upload"
        print "print_ttls"
        sys.exit()
    if cmd == "print_update_waiting_time":
        print_update_waiting_time()
        sys.exit()
    if cmd == "copy_update_waiting_time":
        copy_update_waiting_time()
        sys.exit()
    nr = sys.argv[2]
    if cmd == "desc":
        create_files_desc(nr)
        sys.exit()
    if cmd == "files":
        create_files_from_desc(nr)
        sys.exit()
    if cmd == "kill":
        kill_machines(nr)
        sys.exit()
    if cmd == "upload":
        upload_code(nr)
        sys.exit()
    if cmd == "print_reports":
        print_report(nr)
        sys.exit()
    if cmd == "clear":
        machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
        for m in machines.values():
            clear_data(m)
        os.system("rm -rf " + totalIncommingSizeFileName + "-" + str(nr))
        os.system("rm -rf " + updateWaitingTimeFileName)
        os.system("rm -rf " + periodicEventsFileName + "-" + str(nr))
        os.system("rm -rf " + eventsFileName + "-" + str(nr))
        os.system("rm -rf " + toLeaveFileName + "-" + str(nr))
        sys.exit()
    if cmd == "print_events":
        print_events(nr)
        sys.exit()
    if cmd == "get_logs":
        get_logs(nr)
        sys.exit()
    if cmd == "join":
        num = int(sys.argv[3])
        startMachines(num, nr)
        sys.exit()
    if cmd == "leave":
        num = int(sys.argv[3])
        leaveMachines(num, nr)
        sys.exit()
    if cmd == "print_desc":
        printDesc(nr)
        sys.exit()
    if cmd == "print_report":
        printDesc(nr)
        sys.exit()
    if cmd == "print_incomming_sizes":
        print_incomming_sizes(nr)
        sys.exit()
    if cmd == "print_incomming_sizes2":
        print_incomming_sizes2(nr)
        sys.exit()
    if cmd == "print_ttls":
        print_ttls(nr)
        sys.exit()
    if cmd == "start":
        i = 0
        allIps = ""
        while True:
            i += 1
            configName = configurationFileName + "-" + str(i)
            if not os.path.exists(configName):
                break
            else:
                machines = shelve.open(configName)
                for m in machines.values():
                    allIps += (" " + str(m.hostname))
                    allIps += (" " + str(m.port))
                machines.close()
        machines = shelve.open(configurationFileName + "-" + str(nr), writeback=True)
        machinesProxies = {}
        for m in machines.values():
            machinesProxies[m.machineName] = xmlrpclib.ServerProxy("http://" + str(m.hostname) + ":" + str(m.port), allow_none = True)
        events = shelve.open(eventsFileName + "-" + str(nr), writeback=True)
        filesToBackup = shelve.open(filesToBackupFileName + "-" + str(nr), writeback=True)
        totalIncommingSize = shelve.open(totalIncommingSizeFileName + "-" + str(nr), writeback=True)
        updateWaitingTime = PersistentDict(updateWaitingTimeFileName)
        periodicEvents = shelve.open(periodicEventsFileName + "-" + str(nr), writeback=True)
        ttls = shelve.open(ttlsFileName + "-" + str(nr), writeback=True)
        thread.start_new_thread(
            periodicMonitor,
            (machines, events, nr)
        )

        machinesDict = machines
        machinesProxiesDict = machinesProxies
        eventsDict = events
        periodicEventsDict = periodicEvents
        filesToBackupDict = filesToBackup
        totalIncommingSizeDict = totalIncommingSize
        ttlsDict = ttls
        updateWaitingTimeDict = updateWaitingTime

        try:
            while True:
                periodicWakeup(machines, machinesProxies, events, periodicEvents, filesToBackup, totalIncommingSize, ttls, updateWaitingTime, allIps, nr)
        except KeyboardInterrupt:
            tidyUp()
            sys.exit(1)
        except Exception as e:
            traceback.print_exc()
            print e
            raise e
        sys.exit()
    print "Wrong command name"
