#!/usr/bin/python

import os
import sys
import xmlrpclib
from peerDescriptor import PeerDescriptor
from filesLocationsManager import FilesLocationsManager
import socket
import getpass
from negoServer import startNegoServer
from asynchCommandServer import startAsynchCommandServer
from filesOperations import FilesOperations
from filesOperations import CouldNotMakeBackupException
from filesMetadata import FilesCatalog
from common import KeysManager
from asynchCommandClient import startAsynchCommandDaemon
from tManClient import startLoadBalansingDaemon
from tManServer import startTManServer
import threading
import logs
import thread
import time
import config
from config import ConfigParameters
from SimpleXMLRPCServer import SimpleXMLRPCServer
from ndhtServer import startNDHTServer
from ndhtClient import NDHTClient
from availabilityManager import AvailabilityManager
import logs

class PerformanceParameters:
    def __init__(self):
        self.totalIncommingBwBound = 10 #* 1024 * 1024
        self.desiredBackWindow = 0
        self.myTotalSpace = 100 * 1024 * 1024

class MainFacade:
    def __init__(self, myAddrArg, myPortNumArg, homeDirArg, userPathArg, metadataPath, statsPathArg, ipList, perfParamsArg = PerformanceParameters()):
        self.homeDir = homeDirArg
        self.metadataPath = metadataPath + "/"
        self.__ensureDir(self.metadataPath)

        loggingFilename = self.metadataPath + '/p2pBackupLogs-' + str(myPortNumArg) + '.log'
        logs.init_to_file(loggingFilename, getpass.getuser())
        logs.info("Logs initialized")

        self.config = ConfigParameters()
        self.userPath = userPathArg
        self.myAddr = myAddrArg
        self.myPortNum = myPortNumArg
        self.keysManager = KeysManager(self.myAddr, self.homeDir, self.userPath)
        self.myId = str(self.keysManager.getPeerId())
        self.statsPath = statsPathArg
        self.perfParams = perfParamsArg
        self.serverHttpAddr = "http://" + self.myAddr + ":" + str(int(self.myPortNum) + 3) + "/"

        ipList = [(addr, str(int(port) + 1)) for addr, port in ipList]

        thread.start_new_thread(
            startNDHTServer,
            (self.myAddr, int(self.myPortNum) + 3, self.metadataPath)
        )

        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(int(self.myPortNum) + 3), allow_none = True)
        for i in range(0, 30):
            if i == 29:
                raise Exception()
            try:
                myProxy.isWorking()
                break
            except:
                time.sleep(1)

        self.ndhtClient = None
        thread.start_new_thread(
            self.startNDHTdaemon,
            (ipList, )
        )
        for i in range(0, 30):
            if i == 29:
                raise Exception()
            if self.ndhtClient == None:
                time.sleep(1)
            else:
                break

        self.username = getpass.getuser()
        self.basicPath = os.path.join(self.userPath, "peerstorage")
        self.dataPath = os.path.join(self.basicPath, "data", "")
        self.configPath = os.path.join(self.basicPath, "config", "")

        self.__ensureDir(self.dataPath)
        self.__ensureDir(self.configPath)

        self.filesOperations = FilesOperations(
                                   self.serverHttpAddr,
                                   self.metadataPath,
                                   self.homeDir,
                                   self.userPath,
                                   self.statsPath,
                                   self.myAddr,
                                   self.myPortNum,
                                   self.myId
                               )
        self.availabilityManager = AvailabilityManager(self.config, self.serverHttpAddr, self.myId, self.filesOperations.filesCatalog)
        self.filesLocationsManager = FilesLocationsManager(
                                         self.serverHttpAddr,
                                         self.metadataPath,
                                         self.availabilityManager,
                                         self.filesOperations,
                                         self.myAddr,
                                         self.myPortNum,
                                         self.myId,
                                         self.config,
                                         self.perfParams
                                     )
        self.filesOperations.filesLocationsManager = self.filesLocationsManager

        self.registerNewUser()
        time.sleep(1)
        self.startNegoServer()
        self.startAsynchCommandServer()
        self.startTManServer()

    def startNDHTdaemon(self, ipList):
        self.ndhtClient = NDHTClient(self.config, self.myAddr, int(self.myPortNum), ipList)
        self.ndhtClient.startDaemon()

    def periodicMeasurement(self):
        thread.start_new_thread(self.filesOperations.filesLocationsManager.periodicMeasurement,())

    def periodicAvailabilityMeasurer(self):
        thread.start_new_thread(self.availabilityManager.periodicMonitor,())

    def registerNewUser(self):
        proxy = xmlrpclib.ServerProxy(self.serverHttpAddr, allow_none=True)
        self.keysManager.generateKeys()

        stop = False
        while not stop:
            stop = True
            try:
                try:
                    myPeerDescriptor = PeerDescriptor(self.serverHttpAddr, str(self.myId))
                except xmlrpclib.Fault:
                    myPeerDescriptor = PeerDescriptor()
                    proxy.put(str(self.myId), myPeerDescriptor)
                publicKey = self.keysManager.getPublicKey()
                proxy.putField(str(self.myId), "publicKey", publicKey)
                proxy.putField(str(self.myId), "dataDir", self.userPath)
            except socket.error:
                stop = False
                logs.info("MAIN_FACADE: Trying to connect once again")
                time.sleep(2)

    def leaveNetwork(self):
        proxy = xmlrpclib.ServerProxy(self.serverHttpAddr, allow_none=True)

        #killing my python processes
        myUsername = self.filesOperations.username
        killNegoServer = "pkill -U " + myUsername + " -f .*startNegoServer.*"
        killAsynchCommandServer = "pkill -U " + myUsername + " -f .*startAsynchCommandServer.*"
        killTManServer = "pkill -U " + myUsername + " -f .*startTManServer.*"
        os.system(killNegoServer)
        os.system(killAsynchCommandServer)
        os.system(killTManServer)

    def startNegoServer(self):
        self.periodicMeasurement()
        self.periodicAvailabilityMeasurer()
        thread.start_new_thread(
            startNegoServer,
            (self.serverHttpAddr,
             self.metadataPath,
             self.myId,
             self.username,
             self.myAddr,
             self.myPortNum,
             self.homeDir,
             self.userPath,
             self.statsPath,
             self.config,
             self.filesOperations,
             self.ndhtClient),
            {'dhtAddress': self.serverHttpAddr}
        )
        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(self.myPortNum), allow_none = True)
        for i in range(0, 30):
            if i == 29:
                raise Exception()
            try:
                myProxy.isWorking()
                break
            except:
                time.sleep(1)

    def startAsynchCommandServer(self):
        thread.start_new_thread(
            startAsynchCommandDaemon,
            (self.serverHttpAddr,
             str(self.myId),
             str(self.myAddr),
             self.myPortNum,
             self.statsPath,
             self.config)
        )
        thread.start_new_thread(
            startAsynchCommandServer,
            (self.metadataPath,
             self.myId,
             self.myAddr,
             self.myPortNum,
             self.homeDir,
             self.userPath,
             self.statsPath),
            {'dhtAddress': self.serverHttpAddr}
        )
        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(int(self.myPortNum) + 1), allow_none = True)
        for i in range(0, 30):
            if i == 29:
                raise Exception()
            try:
                myProxy.isWorking()
                break
            except:
                time.sleep(1)

    def startTManServer(self):
        thread.start_new_thread(
            startTManServer,
             (),
             {'myId': self.myId,
              'ipAddr': self.myAddr,
              'port': int(self.myPortNum) + 2,
              'homeDirArg': self.homeDir,
              'userPathArg': self.userPath,
              'statsPathArg': self.statsPath,
              'dhtAddress': self.serverHttpAddr,
              'fileLocationsManager': self.filesLocationsManager,
              'config': self.config}
        )
        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(int(self.myPortNum) + 2), allow_none = True)
        for i in range(0, 30):
            if i == 29:
                raise Exception()
            try:
                myProxy.isWorking()
                break
            except:
                time.sleep(1)

        thread.start_new_thread(
            startLoadBalansingDaemon,
            (self.serverHttpAddr, str(self.myId), self.username, self.myAddr, self.myPortNum, self.config, self.filesLocationsManager)
        )

    def makeBackupAsync(self, fileList):
        thread.start_new_thread(
            self.backupAsyncThread,
            (fileList, )
        )

    def constructData(self, l):
        for e in l:
            filename, size = e
            d = os.path.dirname(filename)
            createDirectoryCommand = "mkdir -p %s" % d
            os.system(createDirectoryCommand)
            createFileCommand = "dd if=/dev/zero of=%s bs=%s count=1" % (filename, str(size))
            os.system(createFileCommand)

    def backupAsyncThread(self, fileList):
        self.constructData(fileList)
        self.makeBackup([filename for filename, size in fileList])

    def backupDirectory(self, dirPath):
        allFiles = []
        for root, dirs, files in os.walk(dirPath):
            for f in files:
                allFiles.append(os.path.join(root, f))
        self.makeBackup(allFiles)

    def makeBackup(self, filesList):
        try:
            self.filesOperations.makeBackup(filesList, True)
            statsFile = open(self.statsPath, 'a')
            statsFile.write("success-backup: 1\n")
            statsFile.close()
        except CouldNotMakeBackupException:
            statsFile = open(self.statsPath, 'a')
            statsFile.write("success-backup: 0\n")
            statsFile.close()

    def printReport(self):
        print "Ratio: " + str(self.getTotalIncommingSize())
        self.filesLocationsManager.printAgreements()

    def printFileVersions(self):
        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(self.myPortNum), allow_none = True)
        return myProxy.printFileVersions()

    def restoreAllData(self):
        self.filesOperations.restore(self.filesOperations.filesCatalog.getAllFileInfo())

    def getAvailability(self):
        return str(self.filesLocationsManager.getAvailability())

    def getTotalIncommingSize(self):
        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(self.myPortNum), allow_none = True)
        freeStorage = int(self.filesLocationsManager.getTotalStorage()) - int(myProxy.getOldAgreementsSize()) - int(myProxy.getAgreementsSize())
        if config.enviroment == "home":
            threshold = 15 * 1024
        else:
            threshold = 70 * 1024 * 1024
        if freeStorage < threshold:
            return -1
        else:
            return str(float(myProxy.getAgreementsSize()) / (self.perfParams.totalIncommingBwBound * self.filesLocationsManager.getAvailability()))

    def getTtls(self):
        return self.filesLocationsManager.getTtls()

    def setTotalIncommingBwBound(self, val):
        self.perfParams.totalIncommingBwBound = val
        self.filesLocationsManager.setTotalIncommingBwBound(val)

    def isWorking(self):
        return 1

    def getFileVersions(self):
        try:
            myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(self.myPortNum), allow_none = True)
            vers = myProxy.getMyFilesVersions()
            return (self.myId, vers)
        except Exception as e:
            logs.trace(e)
            raise e

    def getInfoFromReports(self):
        try:
            myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(self.myPortNum), allow_none = True)
            res = myProxy.getInfoFromReports()
            return res
        except Exception as e:
            logs.trace(e)
            raise e

    def getOthersFilesVersions(self):
        myProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(self.myPortNum), allow_none = True)
        vers = myProxy.getOthersFilesVersions()
        return vers

    def __ensureDir(self, f):
        d = os.path.dirname(f)
        if not os.path.exists(d):
            os.makedirs(d)

def startMainFacade(myAddrArg,
                    myPortNumArg,
                    homeDirArg,
                    userPathArg,
                    metadataPath,
                    statsPathArg,
                    ipList,
                    totalIncommingBwBound,
                    desiredBackWindow,
                    myTotalSpace):
    perfParams = PerformanceParameters()
    perfParams.totalIncommingBwBound = totalIncommingBwBound
    perfParams.desiredBackWindow = desiredBackWindow
    perfParams.myTotalSpace = myTotalSpace

    mainFacade = MainFacade(myAddrArg,
                            myPortNumArg + 1,
                            homeDirArg,
                            userPathArg,
                            metadataPath,
                            statsPathArg,
                            ipList,
                            perfParams)
    
    rpcServer = SimpleXMLRPCServer((myAddrArg, myPortNumArg), allow_none=True, logRequests = False)
    logs.info("MAIN_FACADE: Listening on port "+str(myPortNumArg)+" ...")
    rpcServer.register_instance(mainFacade)
    try:
        rpcServer.serve_forever()
    except:
        logs.close()
        pass

if __name__ == '__main__':
    assert len(sys.argv) >= 10
    myAddr = sys.argv[1]
    myPort = int(sys.argv[2])
    homeDir = sys.argv[3]
    userPath = sys.argv[4]
    statsPath = sys.argv[5]
    totalIncommingBwBound = int(sys.argv[6])
    desiredBackWindow = int(sys.argv[7])
    myTotalSpace = int(sys.argv[8])
    metadataPath = sys.argv[9]
    ipList = []
    for i in range((len(sys.argv) - 10) / 2):
        ipList.append((sys.argv[10 + 2*i], sys.argv[11 + 2*i]))
    startMainFacade(myAddr, myPort, homeDir, userPath, metadataPath, statsPath, ipList, totalIncommingBwBound, desiredBackWindow, myTotalSpace)

