import time
import xmlrpclib
import random
import socket
import logs
from common import QueueWithTimeouts

class NDHTClient(object):

    def __init__(self, config, myAddr, myPort, ipList):
        self.config = config
        self.myAddr = myAddr
        self.myPort = myPort
        self.ipList = ipList
        self.waiting = QueueWithTimeouts()

    def startDaemon(self):
        while True:
            try:
                time.sleep(self.config.ndhtIntervalSec)
                self.periodicWakeup()
            except Exception as e:
                logs.trace(e)

    def updateWithPeer(self, contactAddr, contactPort):
        #logs.debug("NDHT_CLIENT: Exchanging information with peer on address: " + str(contactAddr) + ", port: " + str(contactPort))
        myNDHTProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(int(self.myPort) + 3), allow_none = True)
        di = myNDHTProxy.getDict()
        otherNDHTProxy = xmlrpclib.ServerProxy("http://"+str(contactAddr) + ":" + str(int(contactPort) + 3), allow_none = True)
        otherDi = otherNDHTProxy.getDict()
        myNDHTProxy.updateObjects(otherDi)
        otherNDHTProxy.updateObjects(di)

    def periodicWakeup(self):
        myNDHTProxy = xmlrpclib.ServerProxy("http://"+str(self.myAddr) + ":" + str(int(self.myPort) + 3), allow_none = True)
        di = myNDHTProxy.getDict()
        for val in di.values():
            v, ver = val
            if v["ipAddress"] != None and v["portNumber"] != None:
                p = (v["ipAddress"], v["portNumber"])
                if p not in self.ipList:
                    self.ipList.append(p)
        if ((self.myAddr, self.myPort)) in self.ipList:
            self.ipList.remove((self.myAddr, self.myPort))
        # TODO: potentaially inefficient
        random.shuffle(self.ipList)
        for p in self.ipList:
            if not self.waiting.has(p):
                try:
                    contactAddr, contactPort = p
                    #print "(" + str(self.myAddr) + ", " + str(self.myPort) + ") is contacting (" + str(contactAddr) + ", " + str(contactPort) + ")"
                    self.updateWithPeer(contactAddr, contactPort)
                except socket.error:
                    self.waiting.add(p, self.config.ndhtPenaltyTimeWhenServerNotReachableSec)

