#!/usr/bin/python

import xmlrpclib
import sys
import signal
from SimpleXMLRPCServer import SimpleXMLRPCServer
from persistentDict import PersistentDict
import logs
from time import time
import os

class NDHTServer(object):
    
    def __init__(self, persistentFileName):
        self.storage = PersistentDict(persistentFileName)
        
    def isWorking(self):
        return True

    def put(self, key, value):
        if self.storage.has_key([key]):
            old, version = self.storage.get([key])
            self.storage.set([key], (value, version + 1))
        else:
            self.storage.set([key], (value, 0))
        self.storage.sync()

    def get(self, key):
        obj, ver = self.storage.get([key])
        return obj

    def putField(self, key, field, value):
        obj, ver = self.storage.get([key])
        obj[field] = value
        self.storage.set([key], (obj, ver + 1))
        self.storage.sync()

    def getField(self, key, field):
        obj, ver = self.storage.get([key])
        return obj[field]

    def getAllPeerIds(self):
        self.storage.sync()
        allKeys = self.storage.keys([])
        return allKeys

    def getVersion(self, key):
        obj, ver = self.storage.get([key])
        return ver

    def getDict(self):
        di = {}
        for key in self.storage.keys([]):
            di[key] = self.storage.get([key])
        return di

    def updateObjects(self, di):
        for key, val in di.items():
            obj, ver = val
            if self.storage.has_key([key]):
                obj2, ver2 = self.storage.get([key])
                if ver > ver2:
                    self.storage.set([key], (obj, ver))
                else:
                    self.storage.set([key], (obj2, ver2))
            else:
                self.storage.set([key], (obj, ver))


    def finish(self):
        # shouldn't be accessible through RPC
        self.storage.close()

    def sigfinish(self, signum, frame):
        self.finish()


def startNDHTServer(ipAddr, port, metadataPath):
    persistentFileName = metadataPath + "/ndht-persistent-data-" + str(port)
        
    ndhtServer = NDHTServer(persistentFileName)

    ndhtServer.port = port
    ndhtServer.ipAddress = ipAddr
    
    rpcServer = SimpleXMLRPCServer((ndhtServer.ipAddress, int(ndhtServer.port)), allow_none=True, logRequests = False)
    logs.info("NDHT-SERVER: NDHTServer listening on port "+str(port)+" ...")
    rpcServer.register_instance(ndhtServer)
    try:
        rpcServer.serve_forever()
    except:
        pickleFile = open(persistentFileName, 'wb')
        pickle.dump(ndhtServer, pickleFile)
        pickleFile.close()
