import logging
import xmlrpclib
import socket
import traceback
from peerDescriptor import PeerDescriptor

class NegoClient(object):
    """
    Handles client-side negotation for storage space with some peers
    """
    
    def __init__(self, myPeerId, dhtAddress):
        """
        """
        self.dhtAddress = dhtAddress
        self.dhtProxy = xmlrpclib.ServerProxy(dhtAddress, allow_none=True)
        self.myPeerId = myPeerId
        self.reservedStorage = 2 * 1024 * 1024

    def startReplicate(self, peerId, size):
        """
        Tries to start replicating with peerId; returns same error codes as NegoServer
        """
        try:
            remotePeer = PeerDescriptor(self.dhtAddress, peerId)

            remoteProxy = xmlrpclib.ServerProxy("http://"+str(remotePeer.ipAddress) + ":" + str(remotePeer.portNumber), allow_none = True)
            repResult = remoteProxy.startReplicate(str(self.myPeerId), size)

        except:
            raise

        return repResult

    def authenticateForRestore(self, peerId):
        remotePeer = PeerDescriptor(self.dhtAddress, peerId)
        remoteProxy = xmlrpclib.ServerProxy("http://"+str(remotePeer.ipAddress) + ":" + str(remotePeer.portNumber), allow_none = True)
        remoteProxy.authenticateForRestore(str(self.myPeerId))

    def getFreeStorage(self, peerId):
        remotePeer = PeerDescriptor(self.dhtAddress, peerId)
        remoteProxy = xmlrpclib.ServerProxy("http://"+str(remotePeer.ipAddress) + ":" + str(remotePeer.portNumber), allow_none = True)
        usedStorage = int(remoteProxy.getUsedStorage())
        totalStorage = int(remoteProxy.getTotalStorage())

        return (totalStorage - usedStorage - self.reservedStorage)

    def reserveStorage(self, forId, whereId, size):
        remotePeer = PeerDescriptor(self.dhtAddress, whereId)
        remoteProxy = xmlrpclib.ServerProxy("http://"+str(remotePeer.ipAddress) + ":" + str(remotePeer.portNumber), allow_none = True)
        remoteProxy.reserveStorage(forId, size)

