#!/usr/bin/python

from SimpleXMLRPCServer import SimpleXMLRPCServer
import traceback
import xmlrpclib
import shelve
import config
import sys
import os
import logs
import time
import bsddb

def iterate(listkey, storage):
    listkey.reverse()
    innerMap = storage
    while (len(listkey) > 1):
        k = listkey.pop()
        innerMap = innerMap[k]
    if len(listkey) > 0:
        return (listkey.pop(), innerMap)
    else:
        return (listkey, innerMap)

class ShelveDict:
    def __init__(self, name):
        self.name = name
        self.map = shelve.open(name, writeback=True)

    def reopen(self):
        os.system("rm -f " + str(self.name))
        self.map = shelve.open(self.name, writeback=True)

    def get(self, index):
        try:
            k, m = iterate(index, self.map)
            return m[k]
        except bsddb.db.DBPageNotFoundError as e:
            self.reopen()
            raise e

    def set(self, index, obj):
        try:
            index.reverse()
            innerMap = self.map
            while (len(index) > 1):
                k = index.pop()
                if not innerMap.has_key(k):
                    innerMap[k] = {}
                innerMap = innerMap[k]
            innerMap[index.pop()] = obj
        except bsddb.db.DBPageNotFoundError as e:
            self.reopen()
            raise e

    def remove(self, index):
        try:
            k, m = iterate(index, self.map)
            del m[k]
        except bsddb.db.DBPageNotFoundError as e:
            self.reopen()
            raise e

    def has_key(self, index):
        try:
            k, m = iterate(index, self.map)
            m[k]
            return True
        except bsddb.db.DBPageNotFoundError as e:
            self.reopen()
            raise e
        except:
            return False

    def keys(self, index):
        try:
            k, m = iterate(index, self.map)
            if len(k) == 0:
                return m.keys()
            else:
                return m[k].keys()
        except bsddb.db.DBPageNotFoundError as e:
            self.reopen()
            raise e

    def values(self, index):
        try:
            k, m = iterate(index, self.map)
            if len(k) == 0:
                return m.values()
            else:
                return m[k].values()
        except bsddb.db.DBPageNotFoundError as e:
            self.reopen()
            raise e

    def items(self, index):
        try:
            k, m = iterate(index, self.map)
            if len(k) == 0:
                return m.items()
            else:
                return m[k].items()
        except bsddb.db.DBPageNotFoundError as e:
            self.reopen()
            raise e

    def sync(self):
        self.map.sync()

    def close(self):
        self.map.close()

class ServerDict:
    def __init__(self, name):
        self.name = name
        self.storage = shelve.open(name, writeback=True)
        self.lastSync = time.time()

    def get(self, index):
        try:
            ret = self.storage[index]
        except KeyError:
            self.storage[index] = {}
            self.storage.sync()
            ret = {}
        return ret

    def sync(self, index, newMap, toRemoveMap):
        try:
            ret = self.storage[index]
        except KeyError:
            self.storage[index] = {}
        try:
            for ind, obj in newMap.values():
                k = ind + []
                k.reverse()
                innerMap = self.storage[index]
                while (len(k) > 1):
                    v = k.pop()
                    if not innerMap.has_key(v):
                        innerMap[v] = {}
                    innerMap = innerMap[v]
                innerMap[k.pop()] = obj

            for ind in toRemoveMap.values(): 
                k = ind + []
                try:
                    k1, m = iterate(k, self.storage[index])
                    del m[k1]
                except:
                    pass
            current = time.time()
            if current - self.lastSync > 60:
                try:
                    self.storage.sync()
                    self.lastSync = current
                except:
                    pass
        except Exception as e:
            traceback.print_exc()
            print str(e)
            raise e

class RemoteDict(dict):
    def __init__(self, name):
        self.name = name
        if ("pink" in self.name) or ("orange" in self.name): 
            self.addr = "duch.mimuw.edu.pl"
            self.port = 30007
        elif ("red" in self.name) or ("brown" in self.name): 
            self.addr = "nvidia2"
            self.port = 30007
        elif ("yellow" in self.name) or ("violet" in self.name): 
            self.addr = "duch.mimuw.edu.pl"
            self.port = 30008
        elif ("khaki" in self.name) or ("green" in self.name): 
            self.addr = "duch.mimuw.edu.pl"
            self.port = 30009
        elif ("cyan" in self.name) or ("blue" in self.name): 
            self.addr = "nvidia2"
            self.port = 30008
        elif ("updateWaitingTime" in self.name):
            self.addr = "nvidia2"
            self.port = 30009
        else:
            self.addr = "localhost"
            self.port = 30007
        logs.debug("PERSISTENT_DICT contacting with host listening on port host: " + self.addr + ", and port: " + str(self.port))
        self.proxy = xmlrpclib.ServerProxy("http://"+str(self.addr) + ":" + str(self.port), allow_none = True)
        for i in range(0, 7):
            if i == 6:
                raise Exception()
            try:
                self.localMap = self.proxy.get(self.name)
                break
            except:
                time.sleep(1)
        self.newMap = {}
        self.toRemoveMap = {}

    def get(self, index):
        k, m = iterate(index, self.localMap)
        return m[k]

    def set(self, index, obj):
        index1 = index + []
        index.reverse()
        innerMap = self.localMap
        while (len(index) > 1):
            k = index.pop()
            if not innerMap.has_key(k):
                innerMap[k] = {}
            innerMap = innerMap[k]
        innerMap[index.pop()] = obj

        self.newMap[str(index1)] = (index1, obj)
        try:
            del self.toRemoveMap[str(index1)]
        except:
            pass

    def remove(self, index):
        index1 = index + []
        k, m = iterate(index, self.localMap)
        del m[k]

        self.toRemoveMap[str(index1)] = index1
        try:
            del self.newMap[str(index1)]
        except:
            pass

    def has_key(self, index):
        try:
            k, m = iterate(index, self.localMap)
            m[k]
            return True
        except:
            return False

    def keys(self, index):
        k, m = iterate(index, self.localMap)
        if len(k) == 0:
            return m.keys()
        else:
            return m[k].keys()

    def values(self, index):
        k, m = iterate(index, self.localMap)
        if len(k) == 0:
            return m.values()
        else:
            return m[k].values()

    def items(self, index):
        k, m = iterate(index, self.localMap)
        if len(k) == 0:
            return m.items()
        else:
            return m[k].items()

    def sync(self):
        for i in range(0, 25):
            try:
                self.proxy.sync(self.name, self.newMap, self.toRemoveMap)
                self.newMap.clear()
                self.toRemoveMap.clear()
                break
            except:
                time.sleep(1)


    def close(self):
        pass


def startServer(addr, port, name):
    rpcServer = SimpleXMLRPCServer((addr, port), allow_none=True, logRequests = False)
    storage = ServerDict(name)
    rpcServer.register_instance(storage)
    try:
        rpcServer.serve_forever()
    except:
        pass

if config.enviroment == "lab":
    class PersistentDict(RemoteDict):
        pass
else:
    class PersistentDict(ShelveDict):
        pass

def checkHosts():
    try:
        persistentDict = PersistentDict("pink")
        print "Host duch.mimuw.edu.pl at port 30007 OK"
    except:
        print "Host duch.mimuw.edu.pl at port 30007 unavailable"
    try:
        persistentDict = PersistentDict("red")
        print "Host nvidia2 at port 30007 OK"
    except:
        print "Host nvidia2 at port 30007 unavailable"
    try:
        persistentDict = PersistentDict("yellow")
        print "Host duch.mimuw.edu.pl at port 30008 OK"
    except:
        print "Host duch.mimuw.edu.pl at port 30008 unavailable"
    try:
        persistentDict = PersistentDict("khaki")
        print "Host duch.mimuw.edu.pl at port 30009 OK"
    except:
        print "Host duch.mimuw.edu.pl at port 30009 unavailable"
    try:
        persistentDict = PersistentDict("cyan")
        print "Host nvidia2 at port 30008 OK"
    except:
        print "Host nvidia2 at port 30008 unavailable"
    try:
        persistentDict = PersistentDict("updateWaitingTime")
        print "Host nvidia2 at port 30009 OK"
    except:
        print "Host nvidia2 at port 30009 unavailable"


def repairHosts():
    try:
        persistentDict = PersistentDict("pink")
        print "Host duch.mimuw.edu.pl at port 30007 OK"
    except:
        print "Host duch.mimuw.edu.pl at port 30007 unavailable"
        os.system("ssh -f -o ConnectTimeout=3 ps219737@duch.mimuw.edu.pl pkill -U ps219737 -f persistentDict.py.server.duch.mimuw.edu.pl.30007")
        os.system("ssh -f -o ConnectTimeout=3 ps219737@duch.mimuw.edu.pl 'cd p2pBackup/; python persistentDict.py server duch.mimuw.edu.pl 30007 remoteDict1'")
    try:
        persistentDict = PersistentDict("red")
        print "Host nvidia2 at port 30007 OK"
    except:
        print "Host nvidia2 at port 30007 unavailable"
        os.system("ssh -f -o ConnectTimeout=3 psk@nvidia2 pkill -U psk -f persistentDict.py.server.nvidia2.30007")
        os.system("ssh -f -o ConnectTimeout=3 psk@nvidia2 'cd p2pBackup/; python persistentDict.py server nvidia2 30007 remoteDict1'")
    try:
        persistentDict = PersistentDict("yellow")
        print "Host duch.mimuw.edu.pl at port 30008 OK"
    except:
        print "Host duch.mimuw.edu.pl at port 30008 unavailable"
        os.system("ssh -f -o ConnectTimeout=3 ps219737@duch.mimuw.edu.pl pkill -U ps219737 -f persistentDict.py.server.duch.mimuw.edu.pl.30008")
        os.system("ssh -f -o ConnectTimeout=3 ps219737@duch.mimuw.edu.pl 'cd p2pBackup/; python persistentDict.py server duch.mimuw.edu.pl 30008 remoteDict2'")
    try:
        persistentDict = PersistentDict("khaki")
        print "Host duch.mimuw.edu.pl at port 30009 OK"
    except:
        print "Host duch.mimuw.edu.pl at port 30009 unavailable"
        os.system("ssh -f -o ConnectTimeout=3 ps219737@duch.mimuw.edu.pl pkill -U ps219737 -f persistentDict.py.server.duch.mimuw.edu.pl.30009")
        os.system("ssh -f -o ConnectTimeout=3 ps219737@duch.mimuw.edu.pl 'cd p2pBackup/; python persistentDict.py server duch.mimuw.edu.pl 30009 remoteDict3'")
    try:
        persistentDict = PersistentDict("cyan")
        print "Host nvidia2 at port 30008 OK"
    except:
        print "Host nvidia2 at port 30008 unavailable"
        os.system("ssh -f -o ConnectTimeout=3 psk@nvidia2 pkill -U psk -f persistentDict.py.server.nvidia2.30008")
        os.system("ssh -f -o ConnectTimeout=3 psk@nvidia2 'cd p2pBackup/; python persistentDict.py server nvidia2 30008 remoteDict2'")
    try:
        persistentDict = PersistentDict("updateWaitingTime")
        print "Host nvidia2 at port 30009 OK"
    except:
        print "Host nvidia2 at port 30009 unavailable"
        os.system("ssh -f -o ConnectTimeout=3 psk@nvidia2 pkill -U psk -f persistentDict.py.server.nvidia2.30009")
        os.system("ssh -f -o ConnectTimeout=3 psk@nvidia2 'cd p2pBackup/; python persistentDict.py server nvidia2 30009 remoteDict3'")


def persistentDictTest():
    persistentDict = PersistentDict("some_name")

    persistentDict.set(['a', 'b'], 10)
    persistentDict.set(['a', 'c'], 12)
    persistentDict.set(['d'], 13)
    assert(persistentDict.get(['a'])['b'] == 10)
    assert(persistentDict.get(['a'])['c'] == 12)
    assert(persistentDict.get(['a', 'b']) == 10)
    assert(persistentDict.get(['a', 'c']) == 12)
    assert(persistentDict.get(['d']) == 13)

    assert('a' in persistentDict.keys([]))
    assert('b' not in persistentDict.keys([]))

    assert('b' in persistentDict.keys(['a']))
    assert('c' in persistentDict.keys(['a']))

    assert('a' in persistentDict.keys([]))
    assert('d' in persistentDict.keys([]))

    assert(10 in persistentDict.values(['a']))
    assert(12 in persistentDict.values(['a']))
    assert(len(persistentDict.values([])) == 2)

    assert(persistentDict.has_key(['a']))
    assert(persistentDict.has_key(['d']))
    assert(persistentDict.has_key(['a', 'b']))
    assert(persistentDict.has_key(['a', 'c']))
    assert(not persistentDict.has_key(['d', 'c']))

    persistentDict.sync()
    persistentDict.close()

    persistentDict2 = PersistentDict("some_name")

    assert(persistentDict2.get(['a'])['b'] == 10)
    assert(persistentDict2.get(['a'])['c'] == 12)
    assert(persistentDict2.get(['a', 'b']) == 10)
    assert(persistentDict2.get(['a', 'c']) == 12)
    assert(persistentDict2.get(['d']) == 13)

    assert('a' in persistentDict2.keys([]))
    assert('b' not in persistentDict2.keys([]))

    assert('b' in persistentDict2.keys(['a']))
    assert('c' in persistentDict2.keys(['a']))

    assert('a' in persistentDict2.keys([]))
    assert('d' in persistentDict2.keys([]))

    assert(10 in persistentDict2.values(['a']))
    assert(12 in persistentDict2.values(['a']))
    assert(len(persistentDict2.values([])) == 2)

    assert(persistentDict2.has_key(['a']))
    assert(persistentDict2.has_key(['d']))
    assert(persistentDict2.has_key(['a', 'b']))
    assert(persistentDict2.has_key(['a', 'c']))
    assert(not persistentDict2.has_key(['d', 'c']))

    persistentDict2.remove(['d'])
    assert(persistentDict2.has_key(['a']))
    assert(not persistentDict2.has_key(['d']))
    assert(persistentDict2.has_key(['a', 'b']))
    assert(persistentDict2.has_key(['a', 'c']))

    persistentDict2.remove(['a', 'b'])
    assert(persistentDict2.has_key(['a']))
    assert(not persistentDict2.has_key(['d']))
    assert(not persistentDict2.has_key(['a', 'b']))
    assert(persistentDict2.has_key(['a', 'c']))

    persistentDict2.remove(['a'])
    assert(not persistentDict2.has_key(['a']))
    assert(not persistentDict2.has_key(['d']))

    persistentDict2.sync()
    persistentDict2.close()
    os.system("rm -rf some_name")
    os.system("rm -rf remoteDict")

if __name__ == '__main__':
    cmd = sys.argv[1]
    if cmd == "server":
        host = sys.argv[2]
        port = int(sys.argv[3])
        filename = sys.argv[4]
        startServer(host, port, filename)
    if cmd == "test":
        persistentDictTest()
    if cmd == "check":
        checkHosts()
    if cmd == "repair":
        repairHosts()
