#!/usr/bin/python

import logs
import xmlrpclib
import os
import sys
import pickle
import socket
import getpass
import thread
import threading
import traceback
from SimpleXMLRPCServer import SimpleXMLRPCServer
from xmlrpclib import Fault
from filesOperations import FilesOperations
import xmlrpclib
import random
import time

def peerScoreFromDict(di):
    peerId = di['peerId']
    version = di['version']
    score = di['score']
    pS = PeerScore(peerId, version, score)
    return pS

class PeerScore:
    def __init__(self, peerId, version, score):
        self.peerId = str(peerId)
        self.version = version
        self.score = score

    def __repr__(self):
        s = "id: " + str(self.peerId) + ", version: " + str(self.version) + ", score: " + str(self.score)
        return s

class TManData:
    def __init__(self, myId):
        self.myId = myId
        iniScore = PeerScore(self.myId, 0, 0)
        self.myPeerScore = iniScore
        self.peers = []
        self.randomPeers = []
        self.knownPeers = []
        self.maxPeerNum = 100

    def updateKnownPeers(self, knownPeers):
        for p in knownPeers:
            if p not in self.knownPeers:
                self.knownPeers.append(p)
                self.randomPeers.append(PeerScore(str(p), 0, 0))

    def updateMyScore(self, myScore):
        self.myPeerScore = PeerScore(peerId = self.myId, version = self.myPeerScore.version + 1, score = myScore)

    def updatePeers(self, peerList):
        myList = [self.myPeerScore] + self.peers + self.randomPeers
        mergedList = self.mergeLists(self.mergeLists(self.randomPeers, peerList), self.peers)
        peersByScore = sorted(mergedList, key=lambda pScore: pScore.score)

        myNewList = peersByScore[:self.maxPeerNum]
        self.peers = myNewList
        return myList

    def updateRandom(self, peerList):
        for p in peerList:
            if p not in self.knownPeers:
                self.knownPeers.append(p)
        mList = self.mergeLists(self.randomPeers, peerList)
        size = min(len(mList), self.maxPeerNum)
        result = self.randomPeers + [self.myPeerScore]
        self.randomPeers = random.sample(mList, size)
        return result

    def mergeLists(self, peerList1Arg, peerList2Arg): # this is just merging list which selects newer versions of objects
        peerList1 = peerList1Arg + []
        peerList2 = peerList2Arg + []
        for peer in peerList2:
            if peer.peerId == self.myId:
                peerList2.remove(peer)
        for peer in peerList1:
            append = True
            if peer.peerId != self.myId:
                for peer2 in peerList2:
                    if peer.peerId == peer2.peerId:
                        if peer.version > peer2.version:
                            peerList2.remove(peer2)
                        elif peer.version < peer2.version:
                            append = False
                        elif peer.version == peer2.version:
                            if peer.score == peer2.score:
                                append = False
                if append:
                    peerList2.append(peer)
        return peerList2


class TManServer(object):
    def __init__(self, myIdArg, homeDirArg, userPathArg, statsPathArg, dhtAddress, fileLocationsManager, config):
        self.myPeerId = myIdArg
        self.homeDir = homeDirArg
        self.userPath = userPathArg
        self.statsPath = statsPathArg
        self.tmanData = TManData(self.myPeerId)
        self.fileLocationsManager = fileLocationsManager
        self.dhtAddress = dhtAddress
        self.config = config
        socket.setdefaulttimeout(self.config.socketTimeout)

    def isWorking(self):
        return True

    def tryCommitContracts(self):
        if self.fileLocationsManager.commiting == False:
            thread.start_new_thread(self.fileLocationsManager.tryCommitContracts, ())

    def proposeReplication(self, peerId, peerUsername, size):
        try:
            res = self.fileLocationsManager.proposedReplication(peerId, peerUsername, int(size))
            return res
        except Exception as e:
            logs.trace(e)
            raise(e)

    def updatePerformanceInfo(self, perfInfo, ver):
        try:
            self.fileLocationsManager.updatePerformanceInfo(perfInfo, ver)
        except Exception as e:
            logs.trace(e)
            raise(e)

    def ackDownload(self, filename, peerId, peerUsername):
        self.fileLocationsManager.ackDownload(filename, peerId, peerUsername)

    def getPeersWithLowestScore(self):
        return sorted(self.tmanData.peers, key=lambda pScore: pScore.score)

    def getRandomPeers(self):
        return self.tmanData.randomPeers

    def getMyScore(self):
        return self.tmanData.myPeerScore

    def updatePeers(self, updateInfo):
        self.tmanData.updatePeers(updateInfo)

    def updateRandom(self, randomPeers):
        self.tmanData.updateRandom(randomPeers)

    def updateMyScore(self, myScore):
        self.tmanData.updateMyScore(myScore)

    def updatePeers(self, peerList):
        peerList = [peerScoreFromDict(di) for di in peerList]
        result = self.tmanData.updatePeers(peerList)
        return result

    def updateKnownPeers(self):
        myNDHTProxy = xmlrpclib.ServerProxy(self.dhtAddress, allow_none = True)
        knownPeers = myNDHTProxy.getAllPeerIds()
        self.tmanData.updateKnownPeers(knownPeers)

    def updateRandom(self, peerList):
        peerList = [peerScoreFromDict(di) for di in peerList]
        result = self.tmanData.updateRandom(peerList)
        return result

def startTManServer(myId, ipAddr, port, homeDirArg, userPathArg, statsPathArg, dhtAddress, fileLocationsManager, config):
    tManServer = TManServer(myId, homeDirArg, userPathArg, statsPathArg, dhtAddress, fileLocationsManager, config)
    tManServer.port = port
    tManServer.ipAddress = ipAddr
    tManServer.username = getpass.getuser()
    
    rpcServer = SimpleXMLRPCServer((tManServer.ipAddress, tManServer.port), allow_none=True, logRequests = False)
    logs.info("TMAN_SERVER: Listening on ipAddress: " + str(tManServer.ipAddress) + ", and port " + str(tManServer.port) + " ...")
    rpcServer.register_instance(tManServer)
    try:
        rpcServer.serve_forever()
    except KeyboardInterrupt:
        logs.info("TMAN_SERVER: Closing server")

